import keras
from keras import layers
from keras import ops

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.siglip.siglip_layers import SigLIPHead
from keras_hub.src.models.siglip.siglip_loss import SigLIPLoss


@keras_hub_export("keras_hub.models.SigLIPBackbone")
class SigLIPBackbone(Backbone):
    """SigCLIP core network with hyperparameters.

    This backbone implements the base architecture for the Sigmoid loss in the
    Language-Image Pre-training (SigLIP) model. Unlike standard contrastive
    learning with softmax normalization, the sigmoid loss operates solely on
    image-text pairs and does not require a global view of the pairwise
    similarities for normalization. It includes vision and text encoders. This
    backbone outputs the final logit scores corresponding to each image and
    token input.

    The default constructor gives a fully customizable, randomly initialized
    SigLIP model with any number of layers, heads, and embedding dimensions. To
    load preset architectures and weights, use the `from_preset` constructor.

    Args:
        vision_encoder: The SigLIP vision encoder for encoding the input images.
        text_encoder: The SigLIP text encoder for encoding the input tokens.
        projection_dim: int. The size of the projection layer.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for the models computations and weights. Note that some
            computations, such as softmax and layer normalization will always
            be done a float32 precision regardless of dtype.

    Example:
    ```python
    input_data = {
        "images": np.ones(shape=(1, 224, 224, 3), dtype="float32"),
        "token_ids": np.ones(shape=(1, 64), dtype="int32"),
    }

    # Pretrained SigLIP model.
    model = keras_hub.models.SigLIPBackbone.from_preset(
        "siglip_base_patch16_224"
    )
    model(input_data)

    # Randomly initialized SigLIP model with custom config.
    vision_encoder = keras_hub.models.SigLIPVisionEncoder(
        patch_size=32,
        hidden_dim=768,
        num_layers=8,
        num_heads=8,
        intermediate_dim=2048,
        image_shape=(384, 384, 3),
    )
    text_encoder = keras_hub.models.SigLIPTextEncoder(
        vocabulary_size=32000,
        embedding_dim=768,
        hidden_dim=768,
        num_layers=8,
        num_heads=8,
        intermediate_dim=2048,
    )
    model = keras_hub.models.SigLIPBackbone(
        vision_encoder=vision_encoder,
        text_encoder=text_encoder,
    )
    model(input_data)
    ```
    """

    def __init__(
        self,
        vision_encoder,
        text_encoder,
        dtype=None,
        **kwargs,
    ):
        # === Layers ===
        self.vision_encoder = vision_encoder
        self.text_encoder = text_encoder
        self.siglip_head = SigLIPHead(dtype=dtype, name="siglip_head")

        # === Functional Model ===
        image_input = layers.Input(
            shape=self.vision_encoder.image_shape, name="images"
        )
        token_id_input = layers.Input(
            shape=(None,), dtype="int32", name="token_ids"
        )
        vision_embeddings = self.get_vision_embeddings(image_input)
        text_embeddings = self.get_text_embeddings(token_id_input)
        vision_logits, text_logits = self.siglip_head(
            vision_embeddings, text_embeddings
        )

        super().__init__(
            inputs={
                "images": image_input,
                "token_ids": token_id_input,
            },
            outputs={
                "vision_logits": vision_logits,
                "text_logits": text_logits,
            },
            dtype=dtype,
            **kwargs,
        )

    def compute_loss(
        self, x, y=None, y_pred=None, sample_weight=None, **kwargs
    ):
        outputs = self(x)
        text_logits = outputs["text_logits"]
        batch_size = ops.shape(text_logits)[0]
        eye = ops.eye(batch_size, dtype=text_logits.dtype)
        m1_diag1 = -ops.ones_like(text_logits) + 2 * eye
        return super().compute_loss(
            x=x,
            y=m1_diag1,
            y_pred=text_logits,
            sample_weight=sample_weight,
            **kwargs,
        )

    def compile(
        self,
        optimizer="auto",
        loss="auto",
        metrics=None,
        **kwargs,
    ):
        """Configures the `SigLIPBackbone` task for training.

        `SigLIPBackbone` extends the default compilation signature
        of `keras.Model.compile` with defaults for `optimizer` and `loss`. To
        override these defaults, pass any value to these arguments during
        compilation.

        Args:
            optimizer: `"auto"`, an optimizer name, or a `keras.Optimizer`
                instance. Defaults to `"auto"`, which uses the default
                optimizer for `SigLIPBackbone`. See `keras.Model.compile` and
                `keras.optimizers` for more info on possible `optimizer`
                values.
            loss: `"auto"`, a loss name, or a `keras.losses.Loss` instance.
                Defaults to `"auto"`, in which case the default loss
                computation of `SigLIPBackbone` will be applied.
                See `keras.Model.compile` and `keras.losses` for more info on
                possible `loss` values.
            metrics: `a list of metrics to be evaluated by
                the model during training and testing. Defaults to `None`.
                See `keras.Model.compile` and `keras.metrics` for
                more info on possible `metrics` values.
            **kwargs: See `keras.Model.compile` for a full list of arguments
                supported by the compile method.
        """
        if optimizer == "auto":
            # Using the alternative optimizer AdamW instead of the
            # ScalingViT-Adafactor optimizer mentioned in the paper:
            # https://arxiv.org/abs/2303.15343 - C. Robustness of SigLIP
            # results.
            optimizer = keras.optimizers.AdamW(1e-3, weight_decay=1e-4)
        if loss == "auto":
            loss = SigLIPLoss()
        if metrics == "auto":
            metrics = [keras.metrics.Accuracy()]
        super().compile(
            optimizer=optimizer,
            loss=loss,
            metrics=metrics,
            **kwargs,
        )

    def get_vision_embeddings(self, images):
        """Get the embeddings from the vision encoder.

        Args:
            images: The input tensor for the vision encoder.

        Returns:
            The output embeddings obtained by applying projection layer to the
            pooled output of the vision encoder.
        """
        return self.vision_encoder({"images": images})

    def get_text_embeddings(self, token_ids):
        """Get the embeddings from the text encoder.

        Args:
            token_ids: The input int tensor for the text encoder.

        Returns:
            The output embeddings obtained by applying projection layer to the
            pooled output of the text encoder.
        """
        return self.text_encoder({"token_ids": token_ids})

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "vision_encoder": layers.serialize(self.vision_encoder),
                "text_encoder": layers.serialize(self.text_encoder),
            }
        )
        return config

    @classmethod
    def from_config(cls, config, custom_objects=None):
        config = config.copy()

        # Propagate `dtype` to submodels if needed.
        if "dtype" in config and config["dtype"] is not None:
            dtype_config = config["dtype"]
            if "dtype" not in config["vision_encoder"]["config"]:
                config["vision_encoder"]["config"]["dtype"] = dtype_config
            if "dtype" not in config["text_encoder"]["config"]:
                config["text_encoder"]["config"]["dtype"] = dtype_config

        # We expect submodels to be instantiated.
        config["vision_encoder"] = layers.deserialize(
            config["vision_encoder"], custom_objects=custom_objects
        )
        config["text_encoder"] = layers.deserialize(
            config["text_encoder"], custom_objects=custom_objects
        )
        return cls(**config)



import keras
from keras import activations

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.modeling.position_embedding import PositionEmbedding
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.esm.esm_encoder import ESMEncoder


def esm2_kernel_initializer(stddev=0.02):
    return keras.initializers.TruncatedNormal(stddev=stddev)


@keras_hub_export(
    ["keras_hub.models.ESM2Backbone", "keras_hub.models.ESMBackbone"]
)
class ESMBackbone(Backbone):
    """A ESM2 and ESM encoder network.

    This class implements a bi-directional Transformer-based encoder as
    described in ["ESM"](https://github.com/facebookresearch/esm).

    The default constructor gives a fully customizable, randomly initialized
    ESM2 encoder with any number of layers, heads, and embed dim.To
    load preset architectures and weights, use the `from_preset()` constructor.


    Args:
        vocabulary_size: int. The size of the token vocabulary.
        num_layers: int. The number of transformer layers.
        num_heads: int. The number of attention heads for each transformer.
            The hidden size must be divisible by the number of attention heads.
        hidden_dim: int. The size of the transformer encoding and pooler layers.
        intermediate_dim: int. The output dimension of the first Dense layer in
            a two-layer feedforward network for each transformer.
        dropout: float. Dropout probability for the Transformer encoder.
            Defaults to 0.1
        use_pre_layer_norm:bool.If true, then layer norm will be used before
            entering the transformer block.
            Since it's pre-norm, the default is false.
        max_sequence_length: int. The maximum sequence length that this encoder
            can consume. If None, `max_sequence_length` uses the value from
            sequence length. This determines the variable shape for positional
            embeddings.
        position_embedding_type: str. The position embedding type to use.
            One of "absolute" and "rotary".
            Use "absolute" for ESM1. Use "rotary" for ESM2. Defaults to "rotary"
        max_wavelength : int. The maximum  angular wavelength of
            the sine/cosine curves, for rotary embeddings.
            Defaults to `10000`.
        activation :string or keras.activations. The activation to
            use for the transformer.
            Defaults to `"gelu"`.
        pad_token_id: int.padding token id. Normally 0,
            but is set to 1 in the esm2 model.
            Defaults to 0.
        dtype: None or str or keras.mixed_precision.DTypePolicy. The dtype to
            use for model computations and weights. Note that some computations,
            such as softmax and layer normalization, will always be done at
            float32 precision regardless of dtype.

    Examples:
    ```python
    input_data = {
        "token_ids": np.ones(shape=(1, 12), dtype="int32"),
    }

    # Pretrained ESM2 encoder.
    model = keras_hub.models.ESM2Backbone.from_preset('hf://facebook/esm2_t6_8M_UR50D')
    model(input_data)

    # Randomly initialized ESM2 encoder with a custom config.
    model = keras_hub.models.ESM2Backbone(
        vocabulary_size=30552,
        num_layers=4,
        num_heads=4,
        hidden_dim=256,
        intermediate_dim=512,
    )
    model(input_data)
    ```
    """

    def __init__(
        self,
        vocabulary_size,
        num_layers,
        num_heads,
        hidden_dim,
        intermediate_dim,
        use_bias=True,
        activation="gelu",
        dropout=0.1,
        dtype=None,
        max_sequence_length=1024,
        max_wavelength=10000,
        layer_norm_eps=1e-12,
        use_pre_layer_norm=False,
        position_embedding_type="rotary",
        pad_token_id=0,
        **kwargs,
    ):
        if position_embedding_type not in (
            "rotary",
            "absolute",
        ):
            raise ValueError(
                '`position_embedding_type` must be either `"rotary"`, or '
                '`"absolute"`. Received '
                f"position_embedding_type={position_embedding_type}."
            )
        head_size = hidden_dim // num_heads
        # === Layers ===
        self.token_embedding = keras.layers.Embedding(
            input_dim=vocabulary_size,
            output_dim=hidden_dim,
            embeddings_initializer=esm2_kernel_initializer(),
            dtype=dtype,
            name="token_embedding",
        )
        if position_embedding_type == "absolute":
            self.position_embedding = PositionEmbedding(
                initializer=esm2_kernel_initializer(),
                sequence_length=max_sequence_length,
                dtype=dtype,
                name="position_embedding",
            )
            self.embeddings_add = keras.layers.Add(
                dtype=dtype,
                name="embeddings_add",
            )

        self.output_layer_norm = keras.layers.LayerNormalization(
            epsilon=layer_norm_eps,
            dtype=dtype,
            name="output_layer_norm",
        )
        if use_pre_layer_norm:
            self.emb_layer_norm = keras.layers.LayerNormalization(
                epsilon=layer_norm_eps,
                dtype=dtype,
                name="emb_layer_norm",
            )
        self.transformer_layers = []
        for i in range(num_layers):
            layer = ESMEncoder(
                heads=num_heads,
                head_size=head_size,
                intermediate_size=intermediate_dim,
                use_bias=use_bias,
                max_wavelength=max_wavelength,
                dropout=dropout,
                activation=activation,
                kernel_initializer=esm2_kernel_initializer(),
                layer_norm_eps=layer_norm_eps,
                dtype=dtype,
                use_rotary=position_embedding_type == "rotary",
                name=f"transformer_layer_{i}",
            )
            self.transformer_layers.append(layer)

        # === Functional Model ===
        token_id_input = keras.Input(
            shape=(None,), dtype="int32", name="token_ids"
        )

        attention_mask = keras.ops.not_equal(token_id_input, pad_token_id)

        token_vector = self.token_embedding(token_id_input)
        if position_embedding_type == "absolute":
            position_vector = self.position_embedding(
                token_vector, start_index=pad_token_id
            )
            x = self.embeddings_add([token_vector, position_vector])
        else:
            x = token_vector
        if use_pre_layer_norm:
            x = self.emb_layer_norm(x)
        for transformer_layer in self.transformer_layers:
            x = transformer_layer(x, attention_mask=attention_mask)
        output = self.output_layer_norm(x)
        super().__init__(
            inputs={
                "token_ids": token_id_input,
            },
            outputs=output,
            dtype=dtype,
            **kwargs,
        )

        # === Config ===
        self.vocabulary_size = vocabulary_size
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.hidden_dim = hidden_dim
        self.intermediate_dim = intermediate_dim
        self.dropout = dropout
        self.max_wavelength = max_wavelength
        self.head_size = head_size
        self.activation = activations.get(activation)
        self.use_bias = use_bias
        self.start_token_index = 0
        self.layer_norm_eps = layer_norm_eps
        self.max_sequence_length = max_sequence_length
        self.use_pre_layer_norm = use_pre_layer_norm
        self.position_embedding_type = position_embedding_type
        self.pad_token_id = pad_token_id

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "vocabulary_size": self.vocabulary_size,
                "num_layers": self.num_layers,
                "num_heads": self.num_heads,
                "hidden_dim": self.hidden_dim,
                "intermediate_dim": self.intermediate_dim,
                "dropout": self.dropout,
                "max_wavelength": self.max_wavelength,
                "use_bias": self.use_bias,
                "activation": activations.serialize(self.activation),
                "layer_norm_eps": self.layer_norm_eps,
                "use_pre_layer_norm": self.use_pre_layer_norm,
                "position_embedding_type": self.position_embedding_type,
                "max_sequence_length": self.max_sequence_length,
                "pad_token_id": self.pad_token_id,
            }
        )
        return config



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.densenet.densenet_backbone import DenseNetBackbone
from keras_hub.src.models.densenet.densenet_image_converter import (
    DenseNetImageConverter,
)
from keras_hub.src.models.image_classifier_preprocessor import (
    ImageClassifierPreprocessor,
)


@keras_hub_export("keras_hub.models.DenseNetImageClassifierPreprocessor")
class DenseNetImageClassifierPreprocessor(ImageClassifierPreprocessor):
    backbone_cls = DenseNetBackbone
    image_converter_cls = DenseNetImageConverter



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.image_converter import ImageConverter
from keras_hub.src.models.resnet.resnet_backbone import ResNetBackbone


@keras_hub_export("keras_hub.layers.ResNetImageConverter")
class ResNetImageConverter(ImageConverter):
    backbone_cls = ResNetBackbone



# Metadata for loading pretrained model weights.
backbone_presets = {
    "t5gemma_s_s_ul2": {
        "metadata": {
            "description": (
                "T5Gemma S/S model with a small encoder and small decoder, "
                "adapted as a UL2 model."
            ),
            "params": 312517632,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_s_s_ul2/1",
    },
    "t5gemma_s_s_prefixlm": {
        "metadata": {
            "description": (
                "T5Gemma S/S model with a small encoder and small decoder, "
                "adapted as a prefix language model."
            ),
            "params": 312517632,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_s_s_prefixlm/1",
    },
    "t5gemma_s_s_ul2_it": {
        "metadata": {
            "description": (
                "T5Gemma S/S model with a small encoder and small decoder, "
                "adapted as a UL2 model and fine-tuned for instruction "
                "following."
            ),
            "params": 312517632,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_s_s_ul2_it/1",
    },
    "t5gemma_s_s_prefixlm_it": {
        "metadata": {
            "description": (
                "T5Gemma S/S model with a small encoder and small decoder, "
                "adapted as a prefix language model and fine-tuned for "
                "instruction following."
            ),
            "params": 312517632,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_s_s_prefixlm_it/1",
    },
    "t5gemma_b_b_ul2": {
        "metadata": {
            "description": (
                "T5Gemma B/B model with a base encoder and base decoder, "
                "adapted as a UL2 model."
            ),
            "params": 591490560,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_b_b_ul2/1",
    },
    "t5gemma_b_b_prefixlm": {
        "metadata": {
            "description": (
                "T5Gemma B/B model with a base encoder and base decoder, "
                "adapted as a prefix language model."
            ),
            "params": 591490560,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_b_b_prefixlm/1",
    },
    "t5gemma_b_b_ul2_it": {
        "metadata": {
            "description": (
                "T5Gemma B/B model with a base encoder and base decoder, "
                "adapted as a UL2 model and fine-tuned for instruction "
                "following."
            ),
            "params": 591490560,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_b_b_ul2_it/1",
    },
    "t5gemma_b_b_prefixlm_it": {
        "metadata": {
            "description": (
                "T5Gemma B/B model with a base encoder and base decoder, "
                "adapted as a prefix language model and fine-tuned for "
                "instruction following."
            ),
            "params": 591490560,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_b_b_prefixlm_it/1",
    },
    "t5gemma_l_l_ul2": {
        "metadata": {
            "description": (
                "T5Gemma L/L model with a large encoder and large decoder, "
                "adapted as a UL2 model."
            ),
            "params": 1241761792,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_l_l_ul2/1",
    },
    "t5gemma_l_l_prefixlm": {
        "metadata": {
            "description": (
                "T5Gemma L/L model with a large encoder and large decoder, "
                "adapted as a prefix language model."
            ),
            "params": 1241761792,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_l_l_prefixlm/1",
    },
    "t5gemma_l_l_ul2_it": {
        "metadata": {
            "description": (
                "T5Gemma L/L model with a large encoder and large decoder, "
                "adapted as a UL2 model and fine-tuned for instruction "
                "following."
            ),
            "params": 1241761792,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_l_l_ul2_it/1",
    },
    "t5gemma_l_l_prefixlm_it": {
        "metadata": {
            "description": (
                "T5Gemma L/L model with a large encoder and large decoder, "
                "adapted as a prefix language model and fine-tuned for "
                "instruction following."
            ),
            "params": 1241761792,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_l_l_prefixlm_it/1",
    },
    "t5gemma_ml_ml_ul2": {
        "metadata": {
            "description": (
                "T5Gemma ML/ML model with a medium-large encoder and "
                "medium-large decoder, adapted as a UL2 model."
            ),
            "params": 2200345344,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_ml_ml_ul2/1",
    },
    "t5gemma_ml_ml_prefixlm": {
        "metadata": {
            "description": (
                "T5Gemma ML/ML model with a medium-large encoder and "
                "medium-large decoder, adapted as a prefix language model."
            ),
            "params": 2200345344,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_ml_ml_prefixlm/1",
    },
    "t5gemma_ml_ml_ul2_it": {
        "metadata": {
            "description": (
                "T5Gemma ML/ML model with a medium-large encoder and "
                "medium-large decoder, adapted as a UL2 model and fine-tuned "
                "for instruction following."
            ),
            "params": 2200345344,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_ml_ml_ul2_it/1",
    },
    "t5gemma_ml_ml_prefixlm_it": {
        "metadata": {
            "description": (
                "T5Gemma ML/ML model with a medium-large encoder and "
                "medium-large decoder, adapted as a prefix language model and "
                "fine-tuned for instruction following."
            ),
            "params": 2200345344,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_ml_ml_prefixlm_it/1",
    },
    "t5gemma_xl_xl_ul2": {
        "metadata": {
            "description": (
                "T5Gemma XL/XL model with an extra-large encoder and "
                "extra-large decoder, adapted as a UL2 model."
            ),
            "params": 3766980608,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_xl_xl_ul2/1",
    },
    "t5gemma_xl_xl_prefixlm": {
        "metadata": {
            "description": (
                "T5Gemma XL/XL model with an extra-large encoder and "
                "extra-large decoder, adapted as a prefix language model."
            ),
            "params": 3766980608,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_xl_xl_prefixlm/1",
    },
    "t5gemma_xl_xl_ul2_it": {
        "metadata": {
            "description": (
                "T5Gemma XL/XL model with an extra-large encoder and "
                "extra-large decoder, adapted as a UL2 model and fine-tuned "
                "for instruction following."
            ),
            "params": 3766980608,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_xl_xl_ul2_it/1",
    },
    "t5gemma_xl_xl_prefixlm_it": {
        "metadata": {
            "description": (
                "T5Gemma XL/XL model with an extra-large encoder and "
                "extra-large decoder, adapted as a prefix language model and "
                "fine-tuned for instruction following."
            ),
            "params": 3766980608,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_xl_xl_prefixlm_it/1",
    },
    "t5gemma_2b_2b_ul2": {
        "metadata": {
            "description": (
                "T5Gemma 2B/2B model with a 2-billion-parameter encoder and "
                "2-billion-parameter decoder, adapted as a UL2 model."
            ),
            "params": 5596853760,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_2b_2b_ul2/1",
    },
    "t5gemma_2b_2b_prefixlm": {
        "metadata": {
            "description": (
                "T5Gemma 2B/2B model with a 2-billion-parameter encoder and "
                "2-billion-parameter decoder, adapted as a prefix language "
                "model."
            ),
            "params": 5596853760,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_2b_2b_prefixlm/1",
    },
    "t5gemma_2b_2b_ul2_it": {
        "metadata": {
            "description": (
                "T5Gemma 2B/2B model with a 2-billion-parameter encoder and "
                "2-billion-parameter decoder, adapted as a UL2 model and "
                "fine-tuned for instruction following."
            ),
            "params": 5596853760,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_2b_2b_ul2_it/1",
    },
    "t5gemma_2b_2b_prefixlm_it": {
        "metadata": {
            "description": (
                "T5Gemma 2B/2B model with a 2-billion-parameter encoder and "
                "2-billion-parameter decoder, adapted as a prefix language "
                "model and fine-tuned for instruction following."
            ),
            "params": 5596853760,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_2b_2b_prefixlm_it/1",
    },
    "t5gemma_9b_2b_ul2": {
        "metadata": {
            "description": (
                "T5Gemma 9B/2B model with a 9-billion-parameter encoder and "
                "2-billion-parameter decoder, adapted as a UL2 model."
            ),
            "params": 12292375296,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_9b_2b_ul2/1",
    },
    "t5gemma_9b_2b_prefixlm": {
        "metadata": {
            "description": (
                "T5Gemma 9B/2B model with a 9-billion-parameter encoder and "
                "2-billion-parameter decoder, adapted as a prefix language "
                "model."
            ),
            "params": 12292375296,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_9b_2b_prefixlm/1",
    },
    "t5gemma_9b_2b_ul2_it": {
        "metadata": {
            "description": (
                "T5Gemma 9B/2B model with a 9-billion-parameter encoder and "
                "2-billion-parameter decoder, adapted as a UL2 model and "
                "fine-tuned for instruction following."
            ),
            "params": 12292375296,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_9b_2b_ul2_it/1",
    },
    "t5gemma_9b_2b_prefixlm_it": {
        "metadata": {
            "description": (
                "T5Gemma 9B/2B model with a 9-billion-parameter encoder and "
                "2-billion-parameter decoder, adapted as a prefix language "
                "model and fine-tuned for instruction following."
            ),
            "params": 12292375296,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_9b_2b_prefixlm_it/1",
    },
    "t5gemma_9b_9b_ul2": {
        "metadata": {
            "description": (
                "T5Gemma 9B/9B model with a 9-billion-parameter encoder and "
                "9-billion-parameter decoder, adapted as a UL2 model."
            ),
            "params": 20333401088,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_9b_9b_ul2/1",
    },
    "t5gemma_9b_9b_prefixlm": {
        "metadata": {
            "description": (
                "T5Gemma 9B/9B model with a 9-billion-parameter encoder and "
                "9-billion-parameter decoder, adapted as a prefix language "
                "model."
            ),
            "params": 20333401088,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_9b_9b_prefixlm/1",
    },
    "t5gemma_9b_9b_ul2_it": {
        "metadata": {
            "description": (
                "T5Gemma 9B/9B model with a 9-billion-parameter encoder and "
                "9-billion-parameter decoder, adapted as a UL2 model and "
                "fine-tuned for instruction following."
            ),
            "params": 20333401088,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_9b_9b_ul2_it/1",
    },
    "t5gemma_9b_9b_prefixlm_it": {
        "metadata": {
            "description": (
                "T5Gemma 9B/9B model with a 9-billion-parameter encoder and "
                "9-billion-parameter decoder, adapted as a prefix language "
                "model and fine-tuned for instruction following."
            ),
            "params": 20333401088,
            "path": "t5gemma",
        },
        "kaggle_handle": "kaggle://keras/t5-gemma/keras/t5gemma_9b_9b_prefixlm_it/1",
    },
}



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.parseq.parseq_decoder import PARSeqDecoder


@keras_hub_export("keras_hub.models.PARSeqBackbone")
class PARSeqBackbone(Backbone):
    """Scene Text Detection with PARSeq.

    Performs OCR in natural scenes using the PARSeq model described in [Scene
    Text Recognition with Permuted Autoregressive Sequence Models](
    https://arxiv.org/abs/2207.06966). PARSeq is a ViT-based model that allows
    iterative decoding by performing an autoregressive decoding phase, followed
    by a refinement phase.

    Args:
        image_encoder: keras.Model. The image encoder model.
        vocabulary_size: int. The size of the vocabulary.
        max_label_length: int. The maximum length of the label sequence.
        decoder_hidden_dim: int. The dimension of the decoder hidden layers.
        num_decoder_layers: int. The number of decoder layers.
        num_decoder_heads: int. The number of attention heads in the decoder.
        decoder_mlp_dim: int. The dimension of the decoder MLP hidden layer.
        dropout_rate: float. The dropout rate for the decoder network.
            Defaults to `0.1`.
        attention_dropout: float. The dropout rate for the attention weights.
            Defaults to `0.1`.
        dtype: str. `None`, str, or `keras.mixed_precision.DTypePolicy`. The
            dtype to use for the computations and weights.
        **kwargs: Additional keyword arguments passed to the base
            `keras.Model` constructor.
    """

    def __init__(
        self,
        image_encoder,
        vocabulary_size,
        max_label_length,
        decoder_hidden_dim,
        num_decoder_layers,
        num_decoder_heads,
        decoder_mlp_dim,
        dropout_rate=0.1,
        attention_dropout=0.1,
        dtype=None,
        **kwargs,
    ):
        # === Layers ===
        self.image_encoder = image_encoder
        self.decoder = PARSeqDecoder(
            vocabulary_size=vocabulary_size,
            max_label_length=max_label_length,
            num_layers=num_decoder_layers,
            num_heads=num_decoder_heads,
            hidden_dim=decoder_hidden_dim,
            mlp_dim=decoder_mlp_dim,
            dropout_rate=dropout_rate,
            attention_dropout=attention_dropout,
            name="decoder",
            dtype=dtype,
        )
        self.head = keras.layers.Dense(
            vocabulary_size - 2,  # We don't predict <bos> nor <pad>
            dtype=dtype,
        )

        # === Functional Model ===
        image_input = self.image_encoder.input

        token_id_input = keras.Input(
            shape=(None,), dtype="int32", name="token_ids"
        )
        padding_mask_input = keras.Input(
            shape=(None,), dtype="int32", name="padding_mask"
        )

        memory = self.image_encoder(image_input)
        target_out = self.decoder(
            token_id_input, memory, padding_mask=padding_mask_input
        )
        logits = self.head(target_out)

        # === Config ===
        self.vocabulary_size = vocabulary_size
        self.max_label_length = max_label_length
        self.decoder_hidden_dim = decoder_hidden_dim
        self.num_decoder_layers = num_decoder_layers
        self.num_decoder_heads = num_decoder_heads
        self.decoder_mlp_dim = decoder_mlp_dim
        self.dropout_rate = dropout_rate
        self.attention_dropout = attention_dropout

        super().__init__(
            inputs={
                "images": image_input,
                "token_ids": token_id_input,
                "padding_mask": padding_mask_input,
            },
            outputs=logits,
            dtype=dtype,
            **kwargs,
        )

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "image_encoder": keras.layers.serialize(self.image_encoder),
                "vocabulary_size": self.vocabulary_size,
                "max_label_length": self.max_label_length,
                "decoder_hidden_dim": self.decoder_hidden_dim,
                "num_decoder_layers": self.num_decoder_layers,
                "num_decoder_heads": self.num_decoder_heads,
                "decoder_mlp_dim": self.decoder_mlp_dim,
                "dropout_rate": self.dropout_rate,
                "attention_dropout": self.attention_dropout,
            }
        )

        return config

    @classmethod
    def from_config(cls, config):
        config.update(
            {
                "image_encoder": keras.layers.deserialize(
                    config["image_encoder"]
                ),
            }
        )

        return super().from_config(config)



from keras import layers

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.clip.clip_layers import CLIPEncoderLayer
from keras_hub.src.models.clip.clip_layers import CLIPVisionEmbedding
from keras_hub.src.utils.keras_utils import standardize_data_format


@keras_hub_export("keras_hub.models.CLIPVisionEncoder")
class CLIPVisionEncoder(Backbone):
    """CLIP vision core network with hyperparameters.

    Args:
        patch_size: int. The size of each square patch in the input image.
        hidden_dim: int. The size of the transformer hidden state at the end
            of each transformer layer.
        num_layers: int. The number of transformer layers.
        num_heads: int. The number of attention heads for each transformer.
        intermediate_dim: int. The output dimension of the first Dense layer in
            a two-layer feedforward network for each transformer.
        intermediate_activation: activation function. The activation that
            is used for the first Dense layer in a two-layer feedforward network
            for each transformer.
        intermediate_output_index: optional int. The index of the intermediate
            output. If specified, the output will become a dictionary with two
            keys `"sequence_output"` and `"intermediate_output"`.
        image_shape: tuple. The input shape without the batch size. Defaults to
            `(224, 224, 3)`.
        data_format: `None` or str. If specified, either `"channels_last"` or
            `"channels_first"`. The ordering of the dimensions in the
            inputs. `"channels_last"` corresponds to inputs with shape
            `(batch_size, height, width, channels)`
            while `"channels_first"` corresponds to inputs with shape
            `(batch_size, channels, height, width)`. It defaults to the
            `image_data_format` value found in your Keras config file at
            `~/.keras/keras.json`. If you never set it, then it will be
            `"channels_last"`.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for the models computations and weights. Note that some
            computations, such as softmax and layer normalization will always
            be done a float32 precision regardless of dtype.
    """

    def __init__(
        self,
        patch_size,
        hidden_dim,
        num_layers,
        num_heads,
        intermediate_dim,
        intermediate_activation="quick_gelu",
        intermediate_output_index=None,
        image_shape=(224, 224, 3),
        data_format=None,
        dtype=None,
        name=None,
        **kwargs,
    ):
        data_format = standardize_data_format(data_format)
        if data_format == "channels_last":
            height, width = image_shape[0], image_shape[1]
        else:
            height, width = image_shape[1], image_shape[2]
        if height != width:
            raise ValueError(
                "`CLIPVisionEncoder` expects the height and width to be the "
                f"same in `image_shape`. Received: image_shape={image_shape}"
            )

        if (
            intermediate_output_index is not None
            and intermediate_output_index < 0
        ):
            intermediate_output_index += num_layers

        # `prefix` is used to prevent duplicate name when utilizing multiple
        # CLIP models within a single model, such as in StableDiffusion3.
        prefix = str(name) + "_" if name is not None else ""

        # === Layers ===
        self.embedding = CLIPVisionEmbedding(
            hidden_dim=hidden_dim,
            patch_size=patch_size,
            image_size=height,
            data_format=data_format,
            dtype=dtype,
            name=f"{prefix}embedding",
        )
        self.pre_layer_norm = layers.LayerNormalization(
            epsilon=1e-5, dtype=dtype, name=f"{prefix}pre_layer_norm"
        )
        self.encoder_layers = [
            CLIPEncoderLayer(
                hidden_dim,
                num_heads,
                intermediate_dim,
                intermediate_activation,
                use_causal_mask=False,  # `False` in the vision encoder.
                dtype=dtype,
                name=f"{prefix}encoder_block_{i}",
            )
            for i in range(num_layers)
        ]
        self.layer_norm = layers.LayerNormalization(
            epsilon=1e-5, dtype=dtype, name=f"{prefix}layer_norm"
        )

        # === Functional Model ===
        image_input = layers.Input(shape=image_shape, name="images")
        x = self.embedding(image_input)
        x = self.pre_layer_norm(x)
        intermediate_output = None
        for i, block in enumerate(self.encoder_layers):
            x = block(x)
            if i == intermediate_output_index:
                intermediate_output = x
        sequence_output = self.layer_norm(x)

        if intermediate_output_index is not None:
            outputs = {
                "sequence_output": sequence_output,
                "intermediate_output": intermediate_output,
            }
        else:
            outputs = sequence_output
        super().__init__(
            inputs={"images": image_input},
            outputs=outputs,
            dtype=dtype,
            name=name,
            **kwargs,
        )

        # === Config ===
        self.patch_size = patch_size
        self.hidden_dim = hidden_dim
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.intermediate_dim = intermediate_dim
        self.intermediate_activation = intermediate_activation
        self.intermediate_output_index = intermediate_output_index
        self.image_shape = image_shape

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "patch_size": self.patch_size,
                "hidden_dim": self.hidden_dim,
                "num_layers": self.num_layers,
                "num_heads": self.num_heads,
                "intermediate_dim": self.intermediate_dim,
                "intermediate_activation": self.intermediate_activation,
                "intermediate_output_index": self.intermediate_output_index,
                "image_shape": self.image_shape,
            }
        )
        return config



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_hub.src.samplers.beam_sampler import BeamSampler as BeamSampler
from keras_hub.src.samplers.contrastive_sampler import (
    ContrastiveSampler as ContrastiveSampler,
)
from keras_hub.src.samplers.greedy_sampler import GreedySampler as GreedySampler
from keras_hub.src.samplers.random_sampler import RandomSampler as RandomSampler
from keras_hub.src.samplers.sampler import Sampler as Sampler
from keras_hub.src.samplers.serialization import deserialize as deserialize
from keras_hub.src.samplers.serialization import get as get
from keras_hub.src.samplers.serialization import serialize as serialize
from keras_hub.src.samplers.top_k_sampler import TopKSampler as TopKSampler
from keras_hub.src.samplers.top_p_sampler import TopPSampler as TopPSampler



import keras
from keras import backend

from keras_hub.src.layers.modeling.cached_multi_head_attention import (
    CachedMultiHeadAttention,
)
from keras_hub.src.models.whisper.whisper_cached_multi_head_attention import (
    _build_proj_equation,
)
from keras_hub.src.models.whisper.whisper_cached_multi_head_attention import (
    _get_output_shape,
)


# Removed dependence on einops.
# Source: https://github.com/usefulsensors/moonshine/blob/4a000427bd36a1c2c6d20a86c672dbd850b44c88/moonshine/model.py#L35
def _rotate_half(x):
    """
    Rotates the two halves of the last dimension.

    This function splits the last dimension of the input tensor into two equal
    halves and swaps them with a sign inversion. Specifically, for an input of
    shape `[..., 2*d]`, it returns a tensor of the same shape where `[x1, x2]`
    is transformed into `[-x2, x1]`.

    Args:
        x: Tensor. Shape `[..., 2*d]`. The input tensor to be rotated.

    Returns:
        Tensor: A tensor of shape `[..., 2*d]` with the two halves rotated.
    """
    # Conditional for Tensorflow backend.
    if backend.backend() == "tensorflow":
        x_shape = keras.ops.shape(x)
        last_dim = x_shape[-1]
        d = last_dim // 2
        x_shape_tensor = keras.ops.convert_to_tensor(x_shape)
        new_shape = keras.ops.concatenate(
            [x_shape_tensor[:-1], keras.ops.convert_to_tensor([d, 2])], axis=0
        )
        x = keras.ops.reshape(x, new_shape)
        x1 = x[..., 0]
        x2 = x[..., 1]
        x_rotated = keras.ops.stack([-x2, x1], axis=-1)
        x_rotated = keras.ops.reshape(x_rotated, x_shape)
        return x_rotated

    # Conditional for PyTorch and JAX backends.
    if backend.backend() == "torch" or backend.backend() == "jax":
        x_shape = keras.ops.shape(x)
        x_shape_tuple = tuple(
            int(keras.ops.convert_to_numpy(dim).item()) for dim in x_shape
        )
        last_dim = x_shape_tuple[-1]
        d = last_dim // 2
        new_shape = x_shape_tuple[:-1] + (d, 2)
        x = keras.ops.reshape(x, new_shape)
        x1 = x[..., 0]
        x2 = x[..., 1]
        x_rotated = keras.ops.stack([-x2, x1], axis=-1)
        x_rotated = keras.ops.reshape(x_rotated, x_shape_tuple)
        return x_rotated

    else:
        raise NotImplementedError(
            "Backend not supported. Please use TensorFlow, PyTorch, or JAX."
        )


def _apply_rotary_pos_emb(t, freqs):
    """
    Applies rotary positional embeddings to the input tensor. Used in on-the-fly
    computation of rotary positional embeddings in multi-head attention layers.

    Args:
        t: A tensor with shape `[..., seq_len, ..., hidden_dim]` where the
            rotary embedding is applied to the first `rot_dim` channels of the
            last dimension.
        freqs: A tensor of frequency values with shape `[max_seq_len, rot_dim]`.
            The last `seq_len` entries are used to compute the rotary
            embeddings.

    Returns:
        Tensor: A tensor of the same shape as `t` with the rotary positional
        embeddings applied to the first `rot_dim` channels of the last dimension
        and the remaining channels concatenated unchanged.
    """
    rot_dim = keras.ops.shape(freqs)[-1]
    seq_len = keras.ops.shape(t)[1]
    orig_dtype = t.dtype
    freqs = freqs[:seq_len, :]
    freqs = keras.ops.reshape(freqs, (1, seq_len, 1, rot_dim))
    t_rot = t[..., :rot_dim]
    t_nonrot = t[..., rot_dim:]
    t_rotated = t_rot * keras.ops.cos(freqs) + _rotate_half(
        t_rot
    ) * keras.ops.sin(freqs)
    out = keras.ops.concatenate([t_rotated, t_nonrot], axis=-1)
    return keras.ops.cast(out, orig_dtype)


@keras.saving.register_keras_serializable(package="keras_hub")
class MoonshineMultiHeadAttention(CachedMultiHeadAttention):
    """
    Moonshine multi-head attention layer.

    Implements a multi-head attention mechanism for Moonshine models with
    support for rotary position embeddings and different caching strategies.
    This layer extends the `CachedMultiHeadAttention` base class to include
    specialized functionality for Moonshine models, such as rotary embeddings
    and causal masking.

    Args:
        num_heads: int. Number of attention heads.
        key_dim: int. Size of each attention head for key.
        value_dim: int, optional. Size of each attention head for value. If
            None, defaults to `key_dim`.
        attention_bias: bool, optional. Whether to include bias in attention
            projection layers. Defaults to `False`.
        attention_dropout: float, optional. Dropout probability for attention
            weights. Defaults to 0.0.
        use_causal_mask: bool, optional. Whether to apply causal masking to
            prevent positions from attending to subsequent positions. Defaults
            to `False`.
        apply_rotary_embedding: bool, optional. Whether to apply rotary position
            embeddings to queries and keys. Defaults to `True`.
        **kwargs: Additional keyword arguments passed to the parent class.
    """

    # References:
    # Based on the HuggingFace implementation of the MoonshineAttention class (https://github.com/huggingface/transformers/blob/fc8764c9a618add64c33e83720f974750bcd0978/src/transformers/models/moonshine/modeling_moonshine.py#L184-L315).

    def __init__(
        self,
        num_heads,
        key_dim,
        value_dim=None,
        attention_bias=False,
        attention_dropout=0.0,
        use_causal_mask=False,
        apply_rotary_embedding=True,
        **kwargs,
    ):
        kwargs.pop("use_bias", None)
        kwargs.pop("dropout", None)
        super().__init__(
            num_heads=num_heads,
            key_dim=key_dim,
            value_dim=value_dim,
            use_bias=attention_bias,
            dropout=attention_dropout,
            **kwargs,
        )
        self.attention_bias = attention_bias
        self.attention_dropout = attention_dropout
        self.use_causal_mask = use_causal_mask
        self.apply_rotary_embedding = apply_rotary_embedding

    def build(self, query_shape, value_shape, key_shape=None):
        # Ensure key_shape is defined.
        key_shape = value_shape if key_shape is None else key_shape
        query_rank = len(query_shape)
        value_rank = len(value_shape)
        key_rank = len(key_shape)

        # Build query projection layer.
        einsum_equation, bias_axes, output_rank = _build_proj_equation(
            free_dims=query_rank - 1, bound_dims=1, output_dims=2
        )
        self._query_dense = keras.layers.EinsumDense(
            einsum_equation,
            output_shape=_get_output_shape(
                output_rank - 1, [self._num_heads, self._key_dim]
            ),
            bias_axes=bias_axes if self._use_bias else None,
            name="query",
            **self._get_common_kwargs_for_sublayer(),
        )
        self._query_dense.build(query_shape)

        # Build key projection layer.
        einsum_equation, bias_axes, output_rank = _build_proj_equation(
            free_dims=key_rank - 1, bound_dims=1, output_dims=2
        )
        self._key_dense = keras.layers.EinsumDense(
            einsum_equation,
            output_shape=_get_output_shape(
                output_rank - 1, [self._num_heads, self._key_dim]
            ),
            bias_axes=bias_axes if self._use_bias else None,
            name="key",
            **self._get_common_kwargs_for_sublayer(),
        )
        self._key_dense.build(key_shape)

        # Build value projection layer.
        einsum_equation, bias_axes, output_rank = _build_proj_equation(
            free_dims=value_rank - 1, bound_dims=1, output_dims=2
        )
        self._value_dense = keras.layers.EinsumDense(
            einsum_equation,
            output_shape=_get_output_shape(
                output_rank - 1, [self._num_heads, self._value_dim]
            ),
            bias_axes=bias_axes if self._use_bias else None,
            name="value",
            **self._get_common_kwargs_for_sublayer(),
        )
        self._value_dense.build(value_shape)

        # Build the internal attention computation sublayer.
        self._build_attention(output_rank)

        # Build output projection layer.
        output_shape = (
            query_shape[-1] if not self._output_shape else self._output_shape
        )
        if isinstance(output_shape, (list, tuple)):
            output_shape = list(output_shape)
        else:
            output_shape = [output_shape]

        einsum_equation, bias_axes, output_rank = _build_proj_equation(
            free_dims=query_rank - 1,
            bound_dims=2,
            output_dims=len(output_shape),
        )
        self._output_dense = keras.layers.EinsumDense(
            einsum_equation,
            output_shape=_get_output_shape(output_rank - 1, output_shape),
            bias_axes=bias_axes if self._use_bias else None,
            name="attention_output",
            **self._get_common_kwargs_for_sublayer(),
        )
        output_dense_input_shape = list(
            self._query_dense.compute_output_shape(query_shape)
        )
        output_dense_input_shape[-1] = self._value_dim
        self._output_dense.build(tuple(output_dense_input_shape))

        self.built = True

    def _compute_causal_mask(self, query, value=None, for_cache=False):
        if backend.backend() == "torch" or backend.backend() == "jax":
            q_seq_length = int(
                keras.ops.convert_to_numpy(keras.ops.shape(query)[1]).item()
            )
            v_seq_length = (
                int(
                    keras.ops.convert_to_numpy(keras.ops.shape(value)[1]).item()
                )
                if value is not None
                else q_seq_length
            )
        elif backend.backend() == "tensorflow":
            if for_cache:
                assert value is not None
                v_seq_length = keras.ops.shape(value)[1]
            else:
                v_seq_length = keras.ops.shape(query)[1]
            q_seq_length = keras.ops.shape(query)[1]
        n_rows = v_seq_length if for_cache else q_seq_length
        ones_mask = keras.ops.ones((1, n_rows, v_seq_length), dtype="int32")
        row_index = keras.ops.cumsum(ones_mask, axis=-2)
        col_index = keras.ops.cumsum(ones_mask, axis=-1)
        mask = keras.ops.greater_equal(row_index, col_index)

        if for_cache:
            mask = mask[:, -q_seq_length:, :]

        return mask

    def call(
        self,
        query,
        value,
        key,
        rotary_embedding=None,
        attention_mask=None,
        cache=None,
        cache_update_index=None,
        training=None,
        **kwargs,
    ):
        # Project inputs.
        query_proj = self._query_dense(query)
        if rotary_embedding is not None:
            query_proj = _apply_rotary_pos_emb(query_proj, rotary_embedding)

        # Handle caching.
        if cache is not None:
            key_cache = cache[:, 0, ...]
            value_cache = cache[:, 1, ...]
            if cache_update_index is None:
                key_proj = key_cache
                value_proj = value_cache
            else:
                new_key = self._key_dense(key)
                new_value = self._value_dense(value)
                if self.apply_rotary_embedding and rotary_embedding is not None:
                    new_key = _apply_rotary_pos_emb(new_key, rotary_embedding)
                    update_shape = keras.ops.shape(new_key)
                    start_indices = [0] * len(update_shape)
                    start_indices[1] = cache_update_index
                    key_proj = keras.ops.slice_update(
                        key_cache, tuple(start_indices), new_key
                    )
                    value_proj = keras.ops.slice_update(
                        value_cache, tuple(start_indices), new_value
                    )
                cache = keras.ops.stack((key_proj, value_proj), axis=1)

        else:
            if cache_update_index is not None:
                raise ValueError(
                    "`cache_update_index` should not be set if `cache` is "
                    f"`None`. Received: cache={cache}, cache_update_index="
                    f"{cache_update_index}"
                )
            key_proj = self._key_dense(key)
            value_proj = self._value_dense(value)
            if self.apply_rotary_embedding and rotary_embedding is not None:
                key_proj = _apply_rotary_pos_emb(key_proj, rotary_embedding)

        # Compute attention mask.
        final_mask = attention_mask

        if final_mask is not None:
            mask_shape = keras.ops.shape(final_mask)
            if len(mask_shape) == 2:
                final_mask = final_mask[:, None, None, :]
            elif len(mask_shape) == 3:
                final_mask = final_mask[:, None, :, :]

        attention_kwargs = {
            k: v for k, v in kwargs.items() if k != "padding_mask"
        }
        # Compute attention.
        attention_output, _ = self._compute_attention(
            query=query_proj,
            key=key_proj,
            value=value_proj,
            attention_mask=final_mask,
            training=training,
            **attention_kwargs,
        )

        # Project the attention output.
        output = self._output_dense(attention_output)

        # Return output + cache if cache is provided, otherwise return just
        # output.
        if cache is not None:
            return output, cache
        return output



import math

import keras
from keras import ops

from keras_hub.src.layers.modeling.rotary_embedding import RotaryEmbedding
from keras_hub.src.models.qwen3_moe.qwen3_moe_layernorm import Qwen3MoeLayerNorm
from keras_hub.src.utils.keras_utils import clone_initializer
from keras_hub.src.utils.keras_utils import fused_attention_op_available


class Qwen3MoeAttention(keras.layers.Layer):
    """A multi-head attention layer for Qwen3Moe models
    This attention implementation supports grouped-query attention (GQA) where
    the number of key-value heads can be less than the number of query heads.

    Args:
        num_query_heads: int. Number of query heads.
        num_key_value_heads: int. Number of key/value heads (for GQA).
        head_dim: int. The dimension of each attention head.
        rope_max_wavelength: int. Maximum wavelength for RoPE (Rotary Position
            Embedding).
        rope_scaling_factor: float. Scaling factor for RoPE, used for extending
            context length.
        kernel_initializer: Initializer for the kernel weights.
        dropout: float. Dropout rate for attention weights.
        layer_norm_epsilon: float. The epsilon value for layer normalization.
        sliding_window_size: int. Size of the sliding window for attention.
        **kwargs: Additional keyword arguments to pass to the Layer.
    """

    def __init__(
        self,
        num_query_heads,
        num_key_value_heads,
        head_dim=None,
        rope_max_wavelength=10000,
        rope_scaling_factor=1,
        kernel_initializer="glorot_uniform",
        dropout=0.0,
        layer_norm_epsilon=1e-6,
        sliding_window_size=None,
        **kwargs,
    ):
        super().__init__(
            **kwargs,
        )
        self.num_query_heads = num_query_heads
        self.num_key_value_heads = num_key_value_heads
        self.head_dim = head_dim
        self.dropout = dropout

        self.layer_norm_epsilon = layer_norm_epsilon

        self.num_key_value_groups = num_query_heads // num_key_value_heads
        self.rope_max_wavelength = rope_max_wavelength

        self.kernel_initializer = keras.initializers.get(
            clone_initializer(kernel_initializer)
        )

        self.rope_scaling_factor = rope_scaling_factor
        self.sliding_window_size = sliding_window_size

    def build(self, inputs_shape):
        # Einsum variables:
        # b = batch size
        # q = query length
        # k = key/value length
        # m = model dim
        # u = num query heads
        # v = num key/value heads
        # h = head dim
        hidden_dim = inputs_shape[-1]
        if not self.head_dim:
            self.head_dim = hidden_dim // self.num_query_heads

        self._inv_norm_factor = 1.0 / math.sqrt(self.head_dim)
        self._query_dense = keras.layers.EinsumDense(
            equation="bqm,muh->bquh",
            output_shape=(None, self.num_query_heads, self.head_dim),
            kernel_initializer=self.kernel_initializer,
            dtype=self.dtype_policy,
            name="query",
        )
        self._query_dense.build(inputs_shape)

        self._query_dense_layer_norm = Qwen3MoeLayerNorm(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            head_dim=self.head_dim,
            name="query_dense_layernorm",
        )
        self._query_dense_layer_norm.build(inputs_shape)

        self._key_dense = keras.layers.EinsumDense(
            equation="bkm,mvh->bkvh",
            output_shape=(
                None,
                self.num_key_value_heads,
                self.head_dim,
            ),
            kernel_initializer=self.kernel_initializer,
            dtype=self.dtype_policy,
            name="key",
        )
        self._key_dense.build(inputs_shape)

        self._key_dense_layer_norm = Qwen3MoeLayerNorm(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            head_dim=self.head_dim,
            name="key_dense_layernorm",
        )
        self._key_dense_layer_norm.build(inputs_shape)

        self._value_dense = keras.layers.EinsumDense(
            equation="bkm,mvh->bkvh",
            output_shape=(
                None,
                self.num_key_value_heads,
                self.head_dim,
            ),
            kernel_initializer=self.kernel_initializer,
            dtype=self.dtype_policy,
            name="value",
        )
        self._value_dense.build(inputs_shape)

        self._softmax = keras.layers.Softmax(
            axis=-1,
            dtype="float32",
            name="attention_softmax",
        )

        self._dropout_layer = keras.layers.Dropout(
            rate=self.dropout,
            dtype=self.dtype_policy,
        )

        self._output_dense = keras.layers.EinsumDense(
            equation="bquh,uhm->bqm",
            output_shape=(None, hidden_dim),
            kernel_initializer=self.kernel_initializer,
            dtype=self.dtype_policy,
            name="attention_output",
        )
        self._output_dense.build(
            (None, None, self.num_query_heads, self.head_dim)
        )

        self.rotary_embedding_layer = RotaryEmbedding(
            max_wavelength=self.rope_max_wavelength,
            scaling_factor=self.rope_scaling_factor,
            dtype=self.dtype_policy,
        )

        self._dot_product_equation = "bquh,bkuh->buqk"
        self._combine_equation = "buqk,bkuh->bquh"

        self.built = True

    def call(
        self,
        hidden_states,
        attention_mask=None,
        cache=None,
        cache_update_index=None,
        training=None,
    ):
        """Applies attention mechanism to the input hidden states.

        Args:
            hidden_states: Input tensor of shape [batch_size, seq_length,
                hidden_size].
            attention_mask: Mask tensor of shape [batch_size, seq_length,
                seq_length].
            cache: Optional cached key and value tensors.
            cache_update_index: Index at which to update the cache.
            training: Boolean indicating whether in training mode.

        Returns:
            attention_output: Output tensor after applying attention.
            cache: Updated cache tensors (if cache is provided).
        """
        start_index = (
            cache_update_index if cache_update_index is not None else 0
        )

        query = self._query_dense(hidden_states)
        query = self._query_dense_layer_norm(query)

        # Compute RoPE for queries
        query = self.rotary_embedding_layer(query, start_index=start_index)

        def _compute_key_value(x):
            key = self._key_dense(x)
            key = self._key_dense_layer_norm(key)
            key = self.rotary_embedding_layer(key, start_index=start_index)

            value = self._value_dense(x)

            return key, value

        if cache is not None:
            key_cache = cache[:, 0, ...]
            value_cache = cache[:, 1, ...]
            if cache_update_index is None:
                key = key_cache
                value = value_cache
            else:
                key_update, value_update = _compute_key_value(hidden_states)
                start = [0, cache_update_index, 0, 0]
                key = ops.slice_update(key_cache, start, key_update)
                value = ops.slice_update(value_cache, start, value_update)
                cache = ops.stack((key, value), axis=1)
        else:
            if cache_update_index is not None:
                raise ValueError(
                    "`cache_update_index` should not be set if `cache` is "
                    f"`None`. Received: cache={cache}, "
                    f"cache_update_index={cache_update_index}"
                )
            key, value = _compute_key_value(hidden_states)

        # [batch_shape, seq_len, num_key_value_heads, head_dim]
        # -> [batch_shape, seq_len, num_heads, head_dim]
        key = ops.repeat(key, repeats=self.num_key_value_groups, axis=2)
        value = ops.repeat(value, repeats=self.num_key_value_groups, axis=2)

        attention_output = self._compute_attention(
            query,
            key,
            value,
            attention_mask,
            cache_update_index=cache_update_index,
        )

        attention_output = self._dropout_layer(
            attention_output, training=training
        )

        attention_output = self._output_dense(attention_output)

        if cache is not None:
            return attention_output, cache
        return attention_output

    def _masked_softmax(self, attention_scores, attention_mask=None):
        """Applies softmax with optional masking.

        Args:
            attention_scores: Attention score tensor.
            attention_mask: Optional mask tensor.

        Returns:
            Masked softmax attention weights.
        """
        if attention_mask is not None:
            return self._softmax(
                attention_scores, attention_mask[:, None, :, :]
            )
        return self._softmax(attention_scores)

    def _compute_attention(
        self, query, key, value, attention_mask=None, cache_update_index=None
    ):
        """Computes attention using query, key, and value tensors.
        Uses Flash Attention when available for better performance.

        Args:
            query: Query tensor.
            key: Key tensor.
            value: Value tensor.
            attention_mask: Optional mask tensor.
            cache_update_index: Index for sliding window computation.

        Returns:
            attention_output: Output tensor after applying attention.
        """
        if fused_attention_op_available():
            # Use `dot_product_attention` with Flash Attention support if
            # available.
            if attention_mask is not None:
                attention_mask = ops.expand_dims(attention_mask, axis=1)
                attention_mask = ops.cast(attention_mask, dtype="bool")
            attention_output = ops.dot_product_attention(
                query,
                key,
                value,
                mask=attention_mask,
                scale=self._inv_norm_factor,
            )
            return attention_output

        attention_scores = ops.einsum(self._dot_product_equation, query, key)

        attention_scores = ops.multiply(
            attention_scores,
            ops.cast(self._inv_norm_factor, self.compute_dtype),
        )
        if self.sliding_window_size:
            attention_mask = self._mask_sliding_window(
                attention_mask,
                cache_update_index=cache_update_index
                if cache_update_index is not None
                else 0,
            )
        attention_scores = self._masked_softmax(
            attention_scores, attention_mask
        )
        attention_scores = ops.cast(attention_scores, self.compute_dtype)
        attention_output = ops.einsum(
            self._combine_equation, attention_scores, value
        )

        return attention_output

    def _mask_sliding_window(
        self,
        attention_mask,
        cache_update_index=0,
    ):
        """Creates and combines a sliding window mask with the attention mask.

        Args:
            attention_mask: Original attention mask.
            cache_update_index: Starting index for the sliding window.

        Returns:
            Combined attention mask with sliding window constraints.
        """
        _, query_len, key_len = ops.shape(attention_mask)
        # Compute the sliding window for square attention.
        all_ones = ops.ones((key_len, key_len), "bool")
        if keras.config.backend() == "tensorflow":
            # TODO: trui/tril has issues with dynamic shape on the tensorflow
            # backend. We should fix, but use `band_part` for now.
            import tensorflow as tf

            band_size = ops.minimum(key_len, self.sliding_window_size - 1)
            band_size = ops.cast(band_size, "int32")
            sliding_mask = tf.linalg.band_part(all_ones, band_size, band_size)
        else:
            sliding_mask = ops.triu(
                all_ones, -1 * self.sliding_window_size + 1
            ) * ops.tril(all_ones, self.sliding_window_size - 1)
        # Slice the window for short queries during generation.
        start = (cache_update_index, 0)
        sliding_mask = ops.slice(sliding_mask, start, (query_len, key_len))
        sliding_mask = ops.expand_dims(sliding_mask, 0)
        return ops.logical_and(attention_mask, ops.cast(sliding_mask, "bool"))

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_query_heads": self.num_query_heads,
                "num_key_value_heads": self.num_key_value_heads,
                "rope_max_wavelength": self.rope_max_wavelength,
                "rope_scaling_factor": self.rope_scaling_factor,
                "kernel_initializer": keras.initializers.serialize(
                    self.kernel_initializer
                ),
                "dropout": self.dropout,
                "sliding_window_size": self.sliding_window_size,
                "head_dim": self.head_dim,
                "layer_norm_epsilon": self.layer_norm_epsilon,
            }
        )
        return config



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.hgnetv2.hgnetv2_backbone import HGNetV2Backbone
from keras_hub.src.models.hgnetv2.hgnetv2_image_classifier_preprocessor import (
    HGNetV2ImageClassifierPreprocessor,
)
from keras_hub.src.models.hgnetv2.hgnetv2_layers import HGNetV2ConvLayer
from keras_hub.src.models.image_classifier import ImageClassifier
from keras_hub.src.models.task import Task


@keras_hub_export("keras_hub.models.HGNetV2ImageClassifier")
class HGNetV2ImageClassifier(ImageClassifier):
    """HGNetV2 image classification model.

    `HGNetV2ImageClassifier` wraps a `HGNetV2Backbone` and
    a `HGNetV2ImageClassifierPreprocessor` to create a model that can be used
    for image classification tasks. This model implements the HGNetV2
    architecture with an additional classification head including a 1x1
    convolution layer, global pooling, and a dense output layer.

    The model takes an additional `num_classes` argument, controlling the number
    of predicted output classes, and optionally, a `head_filters` argument to
    specify the number of filters in the classification head's convolution
    layer. To fine-tune with `fit()`, pass a dataset containing tuples of
    `(x, y)` labels where `x` is an image tensor and `y` is an integer from
    `[0, num_classes)`.

    Args:
        backbone: A `HGNetV2Backbone` instance.
        preprocessor: A `HGNetV2ImageClassifierPreprocessor` instance,
            a `keras.Layer` instance, or a callable. If `None` no preprocessing
            will be applied to the inputs.
        num_classes: int. The number of classes to predict.
        head_filters: int, optional. The number of filters in the
            classification head's 1x1 convolution layer. If `None`, it defaults
            to the last value of `hidden_sizes` from the backbone.
        pooling: `"avg"` or `"max"`. The type of global pooling to apply after
            the head convolution. Defaults to `"avg"`.
        activation: `None`, str, or callable. The activation function to use on
            the final `Dense` layer. Set `activation=None` to return the output
            logits. Defaults to `None`.
        dropout: float. Dropout rate applied before the final dense layer.
            Defaults to 0.0.
        head_dtype: `None`, str, or `keras.mixed_precision.DTypePolicy`. The
            dtype to use for the classification head's computations and weights.

    Examples:

    Call `predict()` to run inference.
    ```python
    # Load preset and predict.
    images = np.random.randint(0, 256, size=(2, 224, 224, 3))
    classifier = keras_hub.models.HGNetV2ImageClassifier.from_preset(
        "hgnetv2_b5_ssld_stage2_ft_in1k"
    )
    classifier.predict(images)
    ```

    Call `fit()` on a single batch.
    ```python
    # Load preset and train.
    images = np.random.randint(0, 256, size=(2, 224, 224, 3))
    labels = [0, 3]
    classifier = keras_hub.models.HGNetV2ImageClassifier.from_preset(
        "hgnetv2_b5_ssld_stage2_ft_in1k"
    )
    classifier.fit(x=images, y=labels, batch_size=2)
    ```

    Call `fit()` with custom loss, optimizer and frozen backbone.
    ```python
    classifier = keras_hub.models.HGNetV2ImageClassifier.from_preset(
        "hgnetv2_b5_ssld_stage2_ft_in1k"
    )
    classifier.compile(
        loss=keras.losses.SparseCategoricalCrossentropy(from_logits=True),
        optimizer=keras.optimizers.Adam(5e-5),
    )
    classifier.backbone.trainable = False
    classifier.fit(x=images, y=labels, batch_size=2)
    ```

    Create a custom HGNetV2 classifier with specific head configuration.
    ```python
    backbone = keras_hub.models.HGNetV2Backbone.from_preset(
        "hgnetv2_b5_ssld_stage2_ft_in1k"
    )
    preproc = keras_hub.models.HGNetV2ImageClassifierPreprocessor.from_preset(
        "hgnetv2_b5_ssld_stage2_ft_in1k"
    )
    classifier = keras_hub.models.HGNetV2ImageClassifier(
        backbone=backbone,
        preprocessor=preproc,
        num_classes=10,
        pooling="avg",
        dropout=0.2,
    )
    ```
    """

    backbone_cls = HGNetV2Backbone
    preprocessor_cls = HGNetV2ImageClassifierPreprocessor

    def __init__(
        self,
        backbone,
        preprocessor,
        num_classes,
        head_filters=None,
        pooling="avg",
        activation=None,
        dropout=0.0,
        head_dtype=None,
        **kwargs,
    ):
        name = kwargs.get("name", "hgnetv2_image_classifier")
        head_dtype = head_dtype or backbone.dtype_policy
        data_format = getattr(backbone, "data_format", "channels_last")
        channel_axis = -1 if data_format == "channels_last" else 1
        self.head_filters = (
            head_filters
            if head_filters is not None
            else backbone.hidden_sizes[-1]
        )
        self.activation = activation

        # === Layers ===
        self.backbone = backbone
        self.preprocessor = preprocessor
        self.last_conv = HGNetV2ConvLayer(
            in_channels=backbone.hidden_sizes[-1],
            out_channels=self.head_filters,
            kernel_size=1,
            stride=1,
            groups=1,
            activation="relu",
            use_learnable_affine_block=self.backbone.use_learnable_affine_block,
            data_format=data_format,
            channel_axis=channel_axis,
            name="head_last",
            dtype=head_dtype,
        )
        if pooling == "avg":
            self.pooler = keras.layers.GlobalAveragePooling2D(
                data_format=data_format,
                dtype=head_dtype,
                name=f"{name}_avg_pool" if name else "avg_pool",
            )
        elif pooling == "max":
            self.pooler = keras.layers.GlobalMaxPooling2D(
                data_format=data_format,
                dtype=head_dtype,
                name=f"{name}_max_pool" if name else "max_pool",
            )
        # Check valid pooling.
        else:
            raise ValueError(
                "Unknown `pooling` type. Polling should be either `'avg'` or "
                f"`'max'`. Received: pooling={pooling}."
            )

        self.flatten_layer = keras.layers.Flatten(
            dtype=head_dtype,
            name=f"{name}_flatten" if name else "flatten",
        )
        self.output_dropout = keras.layers.Dropout(
            rate=dropout,
            dtype=head_dtype,
            name=f"{name}_output_dropout" if name else "output_dropout",
        )
        if num_classes > 0:
            self.output_dense = keras.layers.Dense(
                units=num_classes,
                activation=activation,
                dtype=head_dtype,
                name="predictions",
            )
        else:
            self.output_dense = keras.layers.Identity(name="predictions")

        # === Functional Model ===
        inputs = backbone.input
        feature_maps = backbone(inputs)
        last_stage_name = backbone.stage_names[-1]
        last_hidden_state_for_pooling = feature_maps[last_stage_name]
        x = self.last_conv(last_hidden_state_for_pooling)
        x = self.pooler(x)
        x = self.flatten_layer(x)
        x = self.output_dropout(x)
        outputs = self.output_dense(x)
        Task.__init__(
            self,
            inputs=inputs,
            outputs=outputs,
            **kwargs,
        )

        # === Config ===
        self.pooling = pooling
        self.dropout = dropout
        self.num_classes = num_classes

    def get_config(self):
        config = Task.get_config(self)
        config.update(
            {
                "num_classes": self.num_classes,
                "pooling": self.pooling,
                "activation": self.activation,
                "dropout": self.dropout,
                "head_filters": self.head_filters,
            }
        )
        return config



"""ViT model preset configurations."""

# Metadata for loading pretrained model weights.
backbone_presets = {
    "vit_base_patch16_224_imagenet": {
        "metadata": {
            "description": (
                "ViT-B16 model pre-trained on the ImageNet 1k dataset with "
                "image resolution of 224x224 "
            ),
            "params": 85798656,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_base_patch16_224_imagenet/3",
    },
    "vit_base_patch16_384_imagenet": {
        "metadata": {
            "description": (
                "ViT-B16 model pre-trained on the ImageNet 1k dataset with "
                "image resolution of 384x384 "
            ),
            "params": 86090496,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_base_patch16_384_imagenet/3",
    },
    "vit_large_patch16_224_imagenet": {
        "metadata": {
            "description": (
                "ViT-L16 model pre-trained on the ImageNet 1k dataset with "
                "image resolution of 224x224 "
            ),
            "params": 303301632,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_large_patch16_224_imagenet/3",
    },
    "vit_large_patch16_384_imagenet": {
        "metadata": {
            "description": (
                "ViT-L16 model pre-trained on the ImageNet 1k dataset with "
                "image resolution of 384x384 "
            ),
            "params": 303690752,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_large_patch16_384_imagenet/3",
    },
    "vit_base_patch32_384_imagenet": {
        "metadata": {
            "description": (
                "ViT-B32 model pre-trained on the ImageNet 1k dataset with "
                "image resolution of 384x384 "
            ),
            "params": 87528192,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_base_patch32_384_imagenet/2",
    },
    "vit_large_patch32_384_imagenet": {
        "metadata": {
            "description": (
                "ViT-L32 model pre-trained on the ImageNet 1k dataset with "
                "image resolution of 384x384 "
            ),
            "params": 305607680,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_large_patch32_384_imagenet/2",
    },
    "vit_base_patch16_224_imagenet21k": {
        "metadata": {
            "description": (
                "ViT-B16 backbone pre-trained on the ImageNet 21k dataset with "
                "image resolution of 224x224 "
            ),
            "params": 85798656,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_base_patch16_224_imagenet21k/2",
    },
    "vit_base_patch32_224_imagenet21k": {
        "metadata": {
            "description": (
                "ViT-B32 backbone pre-trained on the ImageNet 21k dataset with "
                "image resolution of 224x224 "
            ),
            "params": 87455232,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_base_patch32_224_imagenet21k/2",
    },
    "vit_huge_patch14_224_imagenet21k": {
        "metadata": {
            "description": (
                "ViT-H14 backbone pre-trained on the ImageNet 21k dataset with "
                "image resolution of 224x224 "
            ),
            "params": 630764800,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_huge_patch14_224_imagenet21k/2",
    },
    "vit_large_patch16_224_imagenet21k": {
        "metadata": {
            "description": (
                "ViT-L16 backbone pre-trained on the ImageNet 21k dataset with "
                "image resolution of 224x224 "
            ),
            "params": 303301632,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_large_patch16_224_imagenet21k/2",
    },
    "vit_large_patch32_224_imagenet21k": {
        "metadata": {
            "description": (
                "ViT-L32 backbone pre-trained on the ImageNet 21k dataset with "
                "image resolution of 224x224 "
            ),
            "params": 305510400,
            "path": "vit",
        },
        "kaggle_handle": "kaggle://keras/vit/keras/vit_large_patch32_224_imagenet21k/2",
    },
}



import keras
from keras import ops

from keras_hub.src.utils.keras_utils import standardize_data_format


class DeiTEmbeddings(keras.layers.Layer):
    """Patches the image and embeds the patches.

    Args:
        image_size: tuple. Size of the input image (height, width).
        patch_size: tuple. patch_size: tuple. The size of each image
            patch as (patch_height, patch_width).
        hidden_dim: int. Dimensionality of the patch embeddings.
        num_channels: int. Number of channels in the input image. Defaults to
            `3`.
        data_format: str. `"channels_last"` or `"channels_first"`. Defaults to
            `None` (which uses `"channels_last"`).
        use_mask_token: bool. Whether to use a mask token. Defaults to `False`.
        dropout_rate: float. Dropout rate. Between 0 and 1. Defaults to
            `0.0`.
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(
        self,
        image_size,
        patch_size,
        hidden_dim,
        num_channels=3,
        data_format=None,
        use_mask_token=False,
        dropout_rate=0.0,
        **kwargs,
    ):
        super().__init__(**kwargs)
        num_patches = (image_size[0] // patch_size[0]) * (
            image_size[1] // patch_size[1]
        )
        num_positions = num_patches + 2

        # === Config ===
        self.image_size = image_size
        self.patch_size = patch_size
        self.hidden_dim = hidden_dim
        self.num_channels = num_channels
        self.num_patches = num_patches
        self.num_positions = num_positions
        self.data_format = standardize_data_format(data_format)
        self.use_mask_token = use_mask_token
        self.dropout_rate = dropout_rate

    def build(self, input_shape):
        if self.use_mask_token:
            self.mask_token = self.add_weight(
                shape=(1, 1, self.hidden_dim),
                initializer="zeros",
                dtype=self.variable_dtype,
                name="mask_token",
            )
        self.class_token = self.add_weight(
            shape=(
                1,
                1,
                self.hidden_dim,
            ),
            initializer="zeros",
            dtype=self.variable_dtype,
            name="class_token",
        )
        self.distillation_token = self.add_weight(
            shape=(
                1,
                1,
                self.hidden_dim,
            ),
            initializer="zeros",
            dtype=self.variable_dtype,
            name="distillation_token",
        )
        self.patch_embedding = keras.layers.Conv2D(
            filters=self.hidden_dim,
            kernel_size=self.patch_size,
            strides=self.patch_size,
            padding="valid",
            activation=None,
            dtype=self.dtype_policy,
            data_format=self.data_format,
            name="patch_embedding",
        )
        self.patch_embedding.build(input_shape)
        self.position_embedding = self.add_weight(
            shape=(
                1,
                self.num_positions,
                self.hidden_dim,
            ),  # Matches the shape in PyTorch
            initializer=keras.initializers.RandomNormal(
                stddev=0.02
            ),  # Equivalent to torch.randn()
            dtype=self.variable_dtype,
            trainable=True,
            name="position_embedding",
        )
        self.dropout = keras.layers.Dropout(
            self.dropout_rate, dtype=self.dtype_policy, name="dropout"
        )

        self.built = True

    def call(self, inputs, bool_masked_pos=None):
        patch_embeddings = self.patch_embedding(inputs)
        if self.data_format == "channels_first":
            patch_embeddings = ops.transpose(
                patch_embeddings, axes=(0, 2, 3, 1)
            )
        embeddings_shape = ops.shape(patch_embeddings)
        patch_embeddings = ops.reshape(
            patch_embeddings, [embeddings_shape[0], -1, embeddings_shape[-1]]
        )

        if bool_masked_pos is not None and self.use_mask_token:
            # Expand dimensions to match the embeddings
            bool_masked_pos_expanded = ops.expand_dims(
                bool_masked_pos, axis=-1
            )  # (batch_size, num_patches, 1)
            mask_token_expanded = ops.expand_dims(
                self.mask_token, axis=0
            )  # (1, 1, hidden_size)
            # Apply masking
            embeddings = ops.where(
                bool_masked_pos_expanded, mask_token_expanded, patch_embeddings
            )

        class_token = ops.tile(self.class_token, (embeddings_shape[0], 1, 1))
        distillation_token = ops.tile(
            self.distillation_token, (embeddings_shape[0], 1, 1)
        )
        embeddings = ops.concatenate(
            [class_token, distillation_token, patch_embeddings], axis=1
        )
        position_embedding = self.position_embedding
        embeddings = ops.add(embeddings, position_embedding)
        embeddings = self.dropout(embeddings)
        return embeddings

    def compute_output_shape(self, input_shape):
        return (
            input_shape[0],
            self.num_positions,
            self.hidden_dim,
        )

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "image_size": self.image_size,
                "patch_size": self.patch_size,
                "hidden_dim": self.hidden_dim,
                "num_channels": self.num_channels,
                "num_patches": self.num_patches,
                "num_positions": self.num_positions,
                "use_mask_token": self.use_mask_token,
                "dropout_rate": self.dropout_rate,
            }
        )
        return config


class DeiTIntermediate(keras.layers.Layer):
    """DeiTIntermediate block.
    Args:
        intermediate_dim: int. Dimensionality of the intermediate MLP layer.
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(
        self,
        intermediate_dim,
        **kwargs,
    ):
        super().__init__(**kwargs)

        # === Config ===
        self.intermediate_dim = intermediate_dim

    def build(self, input_shape):
        self.dense = keras.layers.Dense(
            units=self.intermediate_dim,
            activation="gelu",
            dtype=self.dtype_policy,
            name="dense",
        )
        self.dense.build(input_shape)
        self.built = True

    def call(self, inputs):
        out = self.dense(inputs)
        return out

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "intermediate_dim": self.intermediate_dim,
            }
        )
        return config


class DeiTOutput(keras.layers.Layer):
    """DeiT Output layer implementation.
    Args:
        hidden_dim: int. Dimensionality of the patch embeddings.
        dropout_rate: float. Dropout rate. Between 0 and 1. Defaults to
            `0.0`.
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(self, hidden_dim, dropout_rate=0.1, **kwargs):
        super().__init__(**kwargs)
        self.hidden_dim = hidden_dim
        self.dropout_rate = dropout_rate

    def build(self, input_shape):
        self.dense = keras.layers.Dense(
            self.hidden_dim, dtype=self.dtype_policy, name="output"
        )
        self.dense.build(input_shape)

        self.dropout = keras.layers.Dropout(
            self.dropout_rate, dtype=self.dtype_policy, name="dropout"
        )
        # Mark this layer as built
        self.built = True

    def call(self, hidden_states, input_tensor):
        hidden_states = self.dense(hidden_states)  # Linear transformation
        hidden_states = self.dropout(hidden_states)  # Apply dropout
        hidden_states = hidden_states + input_tensor  # Residual connection
        return hidden_states

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "hidden_dim": self.hidden_dim,
                "dropout_rate": self.dropout_rate,
            }
        )
        return config


class DeiTEncoderBlock(keras.layers.Layer):
    """DeiT encoder block.
    Args:
        num_heads: int. Number of attention heads.
        hidden_dim: int. Dimensionality of the hidden representations.
        intermediate_dim: int. Dimensionality of the intermediate MLP layer.
        use_mha_bias: bool. Whether to use bias in the multi-head attention
            layer. Defaults to `True`.
        dropout_rate: float. Dropout rate. Between 0 and 1. Defaults to
            `0.0`.
        attention_dropout: float. Dropout rate for the attention mechanism.
            Between 0 and 1. Defaults to `0.0`.
        layer_norm_epsilon: float. Small float value for layer normalization
            stability. Defaults to `1e-6`.
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(
        self,
        num_heads,
        hidden_dim,
        intermediate_dim,
        use_mha_bias=True,
        dropout_rate=0.0,
        attention_dropout=0.0,
        layer_norm_epsilon=1e-6,
        **kwargs,
    ):
        super().__init__(**kwargs)
        key_dim = hidden_dim // num_heads

        # === Config ===
        self.num_heads = num_heads
        self.hidden_dim = hidden_dim
        self.intermediate_dim = intermediate_dim
        self.key_dim = key_dim
        self.use_mha_bias = use_mha_bias
        self.dropout_rate = dropout_rate
        self.attention_dropout = attention_dropout
        self.layer_norm_epsilon = layer_norm_epsilon

    def build(self, input_shape):
        # Attention block
        self.layer_norm_1 = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
            name="ln_1",
            dtype=self.dtype_policy,
        )
        self.layer_norm_1.build(input_shape)
        self.mha = keras.layers.MultiHeadAttention(
            num_heads=self.num_heads,
            key_dim=self.key_dim,
            use_bias=self.use_mha_bias,
            dropout=self.attention_dropout,
            name="mha",
            dtype=self.dtype_policy,
        )
        self.mha.build(input_shape, input_shape)

        # MLP block
        self.layer_norm_2 = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
            name="ln_2",
            dtype=self.dtype_policy,
        )
        self.layer_norm_2.build((None, None, self.hidden_dim))

        # Intermediate Layer
        self.mlp = DeiTIntermediate(
            self.intermediate_dim, dtype=self.dtype_policy, name="mlp"
        )
        self.mlp.build((None, None, self.hidden_dim))

        # Output Layer
        self.output_layer = DeiTOutput(
            self.hidden_dim,
            self.dropout_rate,
            dtype=self.dtype_policy,
            name="output_layer",
        )

        self.output_layer.build((None, None, self.intermediate_dim))

        self.built = True

    def call(
        self,
        hidden_states,
        attention_mask=None,
        return_attention_scores=False,
    ):
        attention_scores = None
        x = self.layer_norm_1(hidden_states)
        if return_attention_scores:
            x, attention_scores = self.mha(
                x,
                x,
                attention_mask=attention_mask,
                return_attention_scores=return_attention_scores,
            )
        else:
            x = self.mha(
                x,
                x,
                attention_mask=attention_mask,
            )

        x = x + hidden_states
        y = self.layer_norm_2(x)
        y = self.mlp(y)
        y = self.output_layer(y, x)

        return y, attention_scores

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_heads": self.num_heads,
                "hidden_dim": self.hidden_dim,
                "intermediate_dim": self.intermediate_dim,
                "key_dim": self.key_dim,
                "use_mha_bias": self.use_mha_bias,
                "dropout_rate": self.dropout_rate,
                "attention_dropout": self.attention_dropout,
                "layer_norm_epsilon": self.layer_norm_epsilon,
            }
        )
        return config


class DeiTEncoder(keras.layers.Layer):
    """DeiT Encoder class.
    Args:
        num_layers: int. Number of Transformer encoder blocks.
        num_heads: int. Number of attention heads.
        hidden_dim: int. Dimensionality of the hidden representations.
        intermediate_dim: int. Dimensionality of the intermediate MLP layer.
        use_mha_bias: bool. Whether to use bias in the multi-head attention
            layer. Defaults to `True`.
        dropout_rate: float. Dropout rate. Between 0 and 1. Defaults to
            `0.0`.
        attention_dropout: float. Dropout rate for the attention mechanism.
            Between 0 and 1. Defaults to `0.0`.
        layer_norm_epsilon: float. Small float value for layer normalization
            stability. Defaults to `1e-6`.
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(
        self,
        num_layers,
        num_heads,
        hidden_dim,
        intermediate_dim,
        use_mha_bias=True,
        dropout_rate=0.0,
        attention_dropout=0.0,
        layer_norm_epsilon=1e-6,
        **kwargs,
    ):
        super().__init__(**kwargs)

        # === Config ===
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.hidden_dim = hidden_dim
        self.intermediate_dim = intermediate_dim
        self.use_mha_bias = use_mha_bias
        self.dropout_rate = dropout_rate
        self.attention_dropout = attention_dropout
        self.layer_norm_epsilon = layer_norm_epsilon

    def build(self, input_shape):
        self.encoder_layers = []
        for i in range(self.num_layers):
            encoder_block = DeiTEncoderBlock(
                num_heads=self.num_heads,
                hidden_dim=self.hidden_dim,
                intermediate_dim=self.intermediate_dim,
                use_mha_bias=self.use_mha_bias,
                dropout_rate=self.dropout_rate,
                attention_dropout=self.attention_dropout,
                layer_norm_epsilon=self.layer_norm_epsilon,
                dtype=self.dtype_policy,
                name=f"transformer_block_{i + 1}",
            )
            encoder_block.build((None, None, self.hidden_dim))
            self.encoder_layers.append(encoder_block)

        self.layer_norm = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            name="ln",
        )
        self.layer_norm.build((None, None, self.hidden_dim))

        self.built = True

    def call(
        self,
        hidden_states,
        attention_masks=None,
        output_hidden_states=False,
        return_attention_scores=False,
    ):
        seq_len = ops.shape(hidden_states)[1]  # Sequence length
        hidden_dim = ops.shape(hidden_states)[2]  # Hidden size

        # Ensure valid tensor output even if disabled
        all_hidden_states = (
            ops.empty(shape=(0, seq_len, hidden_dim), dtype=hidden_states.dtype)
            if not output_hidden_states
            else ()
        )

        all_self_attentions_scores = (
            ops.empty(
                shape=(0, self.num_heads, seq_len, seq_len),
                dtype=hidden_states.dtype,
            )
            if not return_attention_scores
            else ()
        )

        for i in range(self.num_layers):
            attention_mask = (
                attention_masks[i] if attention_masks is not None else None
            )
            if output_hidden_states:
                all_hidden_states = all_hidden_states + (hidden_states,)

            hidden_states, scores = self.encoder_layers[i](
                hidden_states,
                attention_mask=attention_mask,
                return_attention_scores=return_attention_scores,
            )
            if return_attention_scores:
                all_self_attentions_scores = all_self_attentions_scores + (
                    scores,
                )

        if output_hidden_states:
            all_hidden_states = all_hidden_states + (hidden_states,)

        hidden_states = self.layer_norm(hidden_states)

        return hidden_states, all_hidden_states, all_self_attentions_scores

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_layers": self.num_layers,
                "num_heads": self.num_heads,
                "hidden_dim": self.hidden_dim,
                "intermediate_dim": self.intermediate_dim,
                "key_dim": self.key_dim,
                "use_mha_bias": self.use_mha_bias,
                "dropout_rate": self.dropout_rate,
                "attention_dropout": self.attention_dropout,
                "layer_norm_epsilon": self.layer_norm_epsilon,
            }
        )
        return config



import keras
from keras import ops

from keras_hub.src.layers.modeling.transformer_layer_utils import (
    compute_causal_mask,
)
from keras_hub.src.layers.modeling.transformer_layer_utils import (
    merge_padding_and_attention_mask,
)
from keras_hub.src.models.qwen_moe.qwen_moe_attention import QwenMoeAttention
from keras_hub.src.models.qwen_moe.qwen_moe_layernorm import QwenMoeLayerNorm
from keras_hub.src.utils.keras_utils import clone_initializer


def compute_load_balancing_loss(
    router_logits, num_experts, top_k, attention_mask=None
):
    """
    Compute the load balancing auxiliary loss for a single MoE layer.

    Args:
        router_logits: Tensor of shape (batch_size * seq_len, num_experts).
        num_experts: Integer, total number of experts.
        top_k: Integer, number of experts to select per token.
        attention_mask: Tensor of shape (batch_size, seq_len, seq_len),
            optional mask for padding.

    Returns:
        Scalar tensor representing the auxiliary loss.
    """
    # Compute routing probabilities
    routing_weights = ops.softmax(
        router_logits, axis=-1
    )  # Shape: (batch_size * seq_len, num_experts)

    # Get top-k experts
    _, selected_experts = ops.top_k(
        routing_weights, k=top_k
    )  # Shape: (batch_size * seq_len, top_k)

    # Create one-hot encoding for selected experts
    expert_mask = ops.one_hot(
        selected_experts, num_experts
    )  # Shape: (batch_size * seq_len, top_k, num_experts)

    if attention_mask is not None:
        # Convert attention mask to (batch_size, seq_len)
        batch_size, seq_len, _ = ops.shape(attention_mask)
        flat_mask = ops.any(attention_mask, axis=-1)
        flat_mask = ops.reshape(
            flat_mask, (-1,)
        )  # Shape: (batch_size * seq_len,)
        # Expand mask for broadcasting
        expert_attention_mask = ops.expand_dims(
            flat_mask, axis=-1
        )  # Shape: (batch_size * seq_len, 1)
        expert_attention_mask = ops.cast(expert_attention_mask, dtype="float32")

        # Compute masked means
        tokens_per_expert = ops.sum(
            expert_mask * expert_attention_mask[:, None, :], axis=0
        ) / ops.maximum(
            ops.sum(expert_attention_mask[:, None, :], axis=0), 1e-9
        )  # Shape: (top_k, num_experts)
        router_prob_per_expert = ops.sum(
            routing_weights * expert_attention_mask, axis=0
        ) / ops.maximum(
            ops.sum(expert_attention_mask, axis=0), 1e-9
        )  # Shape: (num_experts,)
    else:
        # Unmasked means
        tokens_per_expert = ops.mean(
            expert_mask, axis=0
        )  # Shape: (top_k, num_experts)
        router_prob_per_expert = ops.mean(
            routing_weights, axis=0
        )  # Shape: (num_experts,)

    # Average over top_k dimension if necessary
    tokens_per_expert = ops.mean(
        tokens_per_expert, axis=0
    )  # Shape: (num_experts,)

    # Compute the loss
    overall_loss = ops.sum(tokens_per_expert * router_prob_per_expert)
    return overall_loss * num_experts


class QwenMoeMLP(keras.layers.Layer):
    def __init__(
        self,
        intermediate_dim,
        hidden_dim,
        activation_fn="silu",
        layer_norm_epsilon=1e-5,
        kernel_initializer="glorot_uniform",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.intermediate_dim = intermediate_dim
        self.hidden_dim = hidden_dim
        self.activation_fn = activation_fn
        self.kernel_initializer = kernel_initializer
        self.layer_norm_epsilon = layer_norm_epsilon

    def build(self, decoder_sequence_shape):
        # Feedforward layers.
        self._feedforward_intermediate_dense = keras.layers.Dense(
            self.intermediate_dim,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            use_bias=False,
            dtype=self.dtype_policy,
            name="feedforward_intermediate_dense",
        )
        self._feedforward_intermediate_dense.build(decoder_sequence_shape)

        self._feedforward_gate_dense = keras.layers.Dense(
            self.intermediate_dim,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            use_bias=False,
            dtype=self.dtype_policy,
            name="feedforward_gate_dense",
        )
        self._feedforward_gate_dense.build(decoder_sequence_shape)

        self._feedforward_output_dense = keras.layers.Dense(
            self.hidden_dim,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            use_bias=False,
            dtype=self.dtype_policy,
            name="feedforward_output_dense",
        )

        self._feedforward_output_dense.build(
            self._feedforward_gate_dense.compute_output_shape(
                decoder_sequence_shape
            )
        )

        self.activation = keras.activations.get(self.activation_fn)
        self.built = True

    def call(self, x):
        gate_output = self._feedforward_gate_dense(x)

        # Note that we run the activation function in full 32-bit
        # precision since this is what `torch.nn.functional.silu`
        # does. Internally, `torch.nn.functional.silu` converts the
        # inputs to float32, computes SiLU, and converts the outputs
        # back to compute dtype.
        # CPU Kernel: https://github.com/pytorch/pytorch/blob/35c493f2cf9b623bfdc7e6b34dc1cb39690a7919/aten/src/ATen/native/cpu/Activation.cpp#L1221-L1235  # noqa: E501
        # CUDA Kernel: https://github.com/pytorch/pytorch/blob/35c493f2cf9b623bfdc7e6b34dc1cb39690a7919/aten/src/ATen/native/cuda/ActivationSiluKernel.cu  # noqa: E501
        gate_output = ops.cast(gate_output, "float32")
        gate_output = self.activation(gate_output)
        gate_output = ops.cast(gate_output, self.compute_dtype)

        x = self._feedforward_intermediate_dense(x)

        x = self._feedforward_output_dense(ops.multiply(x, gate_output))

        return x


class QwenMoeExperts(keras.layers.Layer):
    """Batched Experts Layer"""

    def __init__(
        self,
        num_experts,
        hidden_dim,
        intermediate_dim,
        activation_fn="silu",
        kernel_initializer="glorot_uniform",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.num_experts = num_experts
        self.hidden_dim = hidden_dim
        self.intermediate_dim = intermediate_dim
        self.activation = keras.activations.get(activation_fn)
        self.kernel_initializer = kernel_initializer

    def build(self, _):
        self._expert_feedforward_gate_dense = self.add_weight(
            shape=(
                self.num_experts,
                self.hidden_dim,
                2 * self.intermediate_dim,
            ),
            initializer=self.kernel_initializer,
            trainable=True,
            dtype=self.variable_dtype,
            name="expert_feedforward_gate_dense",
        )

        self._expert_feedforward_output_dense = self.add_weight(
            shape=(self.num_experts, self.intermediate_dim, self.hidden_dim),
            initializer=self.kernel_initializer,
            trainable=True,
            dtype=self.variable_dtype,
            name="expert_feedforward_output_dense",
        )

        self.built = True

    def call(self, hidden_states):
        gate_up = ops.einsum(
            "th,ehm->etm", hidden_states, self._expert_feedforward_gate_dense
        )
        gate, up = ops.split(gate_up, 2, axis=-1)
        hidden = up * self.activation(gate)
        out = ops.einsum(
            "eti,eih->eth", hidden, self._expert_feedforward_output_dense
        )
        return out


class QwenSparseMoeBlock(keras.layers.Layer):
    """Qwen-2 Sparse Moe Block"""

    def __init__(
        self,
        hidden_dim,
        moe_intermediate_dim,
        shared_expert_intermediate_dim,
        num_experts,
        top_k,
        norm_top_k_prob,
        kernel_initializer="glorot_uniform",
        layer_norm_epsilon=1e-5,
        router_aux_loss_coefficient=0.01,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.hidden_dim = hidden_dim
        self.intermediate_dim = moe_intermediate_dim
        self.intermediate_dim_shared = shared_expert_intermediate_dim
        self.num_experts = num_experts
        self.top_k = top_k
        self.norm_top_k_prob = norm_top_k_prob
        self.kernel_initializer = kernel_initializer
        self.layer_norm_epsilon = layer_norm_epsilon
        self.router_aux_loss_coefficient = router_aux_loss_coefficient

    def build(self, decoder_sequence_shape):
        self._sparse_feedforward_gate_dense = keras.layers.Dense(
            self.num_experts,
            use_bias=False,
            kernel_initializer=self.kernel_initializer,
            name="sparse_feedforward_gate_dense",
            dtype=self.dtype_policy,
        )
        self._sparse_feedforward_gate_dense.build(decoder_sequence_shape)

        # NOTE: Experts are implemented as a single layer to enable efficient
        # batched computation. Implementing each expert individually is
        # currently avoided due to the lack of `ragged_dot` support in the
        # Keras ops API, which would make individual implementations unstable
        # and prone to bugs.
        self.expert_bank = QwenMoeExperts(
            num_experts=self.num_experts,
            hidden_dim=self.hidden_dim,
            intermediate_dim=self.intermediate_dim,
            kernel_initializer=self.kernel_initializer,
            name="experts",
            dtype=self.dtype_policy,
        )
        self.expert_bank.build(decoder_sequence_shape)

        self.shared_expert_dense = QwenMoeMLP(
            intermediate_dim=self.intermediate_dim_shared,
            hidden_dim=self.hidden_dim,
            kernel_initializer=self.kernel_initializer,
            layer_norm_epsilon=self.layer_norm_epsilon,
            name="shared_expert_dense",
            dtype=self.dtype_policy,
        )
        self.shared_expert_dense.build(decoder_sequence_shape)

        self.shared_expert_gate_dense = keras.layers.Dense(
            1,
            use_bias=False,
            name="shared_expert_gate_dense",
            dtype=self.dtype_policy,
        )
        self.shared_expert_gate_dense.build(decoder_sequence_shape)

        self.built = True

    def call(self, hidden_states, attention_mask=None, training=None):
        batch_size, seq_len, _ = ops.shape(hidden_states)
        hidden_states_flattened = ops.reshape(
            hidden_states, (-1, self.hidden_dim)
        )

        router_logits = self._sparse_feedforward_gate_dense(
            hidden_states_flattened
        )
        router_probs = ops.softmax(router_logits, axis=-1)

        top_p, top_i = ops.top_k(router_probs, k=self.top_k)
        if self.norm_top_k_prob:
            top_p = top_p / ops.sum(top_p, axis=-1, keepdims=True)

        one_hot = ops.one_hot(top_i, self.num_experts)
        one_hot = ops.cast(one_hot, top_p.dtype)
        routing_full = ops.sum(one_hot * top_p[..., None], axis=1)
        routing_full = ops.transpose(routing_full, (1, 0))
        routing_full = ops.cast(routing_full, hidden_states_flattened.dtype)

        expert_out = self.expert_bank(hidden_states_flattened)

        weighted_out = expert_out * routing_full[:, :, None]
        expert_contribution = ops.sum(weighted_out, axis=0)

        shared_expert_output = self.shared_expert_dense(hidden_states_flattened)
        shared_expert_output *= ops.sigmoid(
            self.shared_expert_gate_dense(hidden_states_flattened)
        )

        out_flat = expert_contribution + shared_expert_output
        out = ops.reshape(out_flat, (batch_size, seq_len, self.hidden_dim))

        # Compute and add auxiliary loss during training
        if training:
            aux_loss = compute_load_balancing_loss(
                router_logits=router_logits,
                num_experts=self.num_experts,
                top_k=self.top_k,
                attention_mask=attention_mask,
            )
            self.add_loss(self.router_aux_loss_coefficient * aux_loss)

        return out, router_logits


class QwenMoeTransformerDecoder(keras.layers.Layer):
    def __init__(
        self,
        intermediate_dim,
        num_query_heads,
        num_key_value_heads,
        moe_intermediate_dim,
        shared_expert_intermediate_dim,
        num_experts,
        top_k,
        norm_top_k_prob,
        decoder_sparse_step,
        rope_max_wavelength=10000,
        rope_scaling_factor=1.0,
        activation="silu",
        layer_norm_epsilon=1e-5,
        kernel_initializer="glorot_uniform",
        dropout=0,
        use_sliding_window_attention=False,
        sliding_window_size=4096,
        layer_index=0,
        mlp_only_layers=[],
        output_router_logits=False,
        router_aux_loss_coefficient=0.001,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.intermediate_dim = intermediate_dim
        self.num_query_heads = num_query_heads
        self.num_key_value_heads = num_key_value_heads
        self.rope_max_wavelength = rope_max_wavelength
        self.rope_scaling_factor = rope_scaling_factor
        self.dropout = dropout
        self.use_sliding_window_attention = use_sliding_window_attention
        self.sliding_window_size = sliding_window_size
        self.activation = keras.activations.get(activation)
        self.layer_norm_epsilon = layer_norm_epsilon
        self.kernel_initializer = keras.initializers.get(kernel_initializer)
        self.layer_index = layer_index
        self.mlp_only_layers = mlp_only_layers
        self.moe_intermediate_dim = moe_intermediate_dim
        self.shared_expert_intermediate_dim = shared_expert_intermediate_dim
        self.num_experts = num_experts
        self.top_k = top_k
        self.norm_top_k_prob = norm_top_k_prob
        self.decoder_sparse_step = decoder_sparse_step
        self.output_router_logits = output_router_logits
        self.router_aux_loss_coefficient = router_aux_loss_coefficient
        self.supports_masking = True

    def build(self, decoder_sequence_shape):
        self._decoder_sequence_shape = decoder_sequence_shape
        self.hidden_dim = decoder_sequence_shape[-1]

        # Self attention layer.
        self._self_attention_layer = QwenMoeAttention(
            num_query_heads=self.num_query_heads,
            num_key_value_heads=self.num_key_value_heads,
            rope_max_wavelength=self.rope_max_wavelength,
            rope_scaling_factor=self.rope_scaling_factor,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            dropout=self.dropout,
            use_sliding_window_attention=self.use_sliding_window_attention,
            sliding_window_size=self.sliding_window_size,
            name="self_attention",
            dtype=self.dtype_policy,
        )
        self._self_attention_layer.build(decoder_sequence_shape)

        self._self_attention_layernorm = QwenMoeLayerNorm(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            name="self_attention_layernorm",
        )

        self._self_attention_layernorm.build(decoder_sequence_shape)
        self._self_attention_dropout = keras.layers.Dropout(
            rate=self.dropout,
            dtype=self.dtype_policy,
            name="self_attention_dropout",
        )

        # Feedforward layers.
        if (self.layer_index not in self.mlp_only_layers) and (
            self.num_experts > 0
            and (self.layer_index + 1) % self.decoder_sparse_step == 0
        ):
            self.mlp = QwenSparseMoeBlock(
                hidden_dim=self.hidden_dim,
                moe_intermediate_dim=self.moe_intermediate_dim,
                shared_expert_intermediate_dim=self.shared_expert_intermediate_dim,
                num_experts=self.num_experts,
                top_k=self.top_k,
                norm_top_k_prob=self.norm_top_k_prob,
                router_aux_loss_coefficient=self.router_aux_loss_coefficient,
                kernel_initializer=self.kernel_initializer,
                dtype=self.dtype_policy,
            )
            self.mlp.build(decoder_sequence_shape)
        else:
            self.mlp = QwenMoeMLP(
                intermediate_dim=self.intermediate_dim,
                hidden_dim=self.hidden_dim,
                dtype=self.dtype_policy,
            )
            self.mlp.build(decoder_sequence_shape)

        self._feedforward_layernorm = QwenMoeLayerNorm(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            name="feedforward_layernorm",
        )
        self._feedforward_layernorm.build(decoder_sequence_shape)

        self.built = True

    def call(
        self,
        decoder_sequence,
        decoder_padding_mask=None,
        decoder_attention_mask=None,
        self_attention_cache=None,
        self_attention_cache_update_index=None,
        training=None,
    ):
        """Forward pass for the decoder layer.

        Args:
            decoder_sequence: Input tensor of shape [batch_size, seq_length,
                hidden_size].
            decoder_padding_mask: Mask tensor for padding tokens.
            decoder_attention_mask: Additional attention mask.
            self_attention_cache: Optional cached key and value tensors for
                self-attention.
            self_attention_cache_update_index: Index at which to update the
                cache.
            training: Boolean indicating whether in training mode.

        Returns:
            decoder_output: Output tensor after applying transformer decoder
                block.
            self_attention_cache: Updated cache tensors (if cache is provided).
        """
        self_attention_mask = self._compute_self_attention_mask(
            decoder_sequence=decoder_sequence,
            decoder_padding_mask=decoder_padding_mask,
            decoder_attention_mask=decoder_attention_mask,
            self_attention_cache=self_attention_cache,
            self_attention_cache_update_index=self_attention_cache_update_index,
        )
        residual = decoder_sequence

        x = self._self_attention_layernorm(decoder_sequence)

        # Self attention block.
        x = self._self_attention_layer(
            hidden_states=x,
            attention_mask=self_attention_mask,
            cache=self_attention_cache,
            cache_update_index=self_attention_cache_update_index,
        )

        if self_attention_cache is not None:
            x, self_attention_cache = x

        x = self._self_attention_dropout(x, training=training)

        x = x + residual
        residual = x

        x = self._feedforward_layernorm(x)
        if isinstance(self.mlp, QwenSparseMoeBlock):
            x = self.mlp(
                x, training=training, attention_mask=self_attention_mask
            )
        else:
            x = self.mlp(x)
        if isinstance(x, tuple):
            x, router_logits = x
        else:
            router_logits = None

        x = ops.cast(x, ops.dtype(residual))
        decoder_output = x + residual

        output = (decoder_output,)

        if self_attention_cache is not None:
            output += (self_attention_cache,)

        if self.output_router_logits:
            output += (router_logits,)

        return output[0] if len(output) == 1 else output

    def _compute_self_attention_mask(
        self,
        decoder_sequence,
        decoder_padding_mask,
        decoder_attention_mask,
        self_attention_cache,
        self_attention_cache_update_index,
    ):
        """Computes the self-attention mask combining causal, padding and
        attention masks.

        Args:
            decoder_sequence: Input tensor.
            decoder_padding_mask: Mask tensor for padding tokens.
            decoder_attention_mask: Additional attention mask.
            self_attention_cache: Optional cached key and value tensors.
            self_attention_cache_update_index: Index at which to update the
                cache.

        Returns:
            Combined attention mask tensor.
        """
        decoder_mask = merge_padding_and_attention_mask(
            decoder_sequence, decoder_padding_mask, decoder_attention_mask
        )
        batch_size = ops.shape(decoder_sequence)[0]
        input_length = output_length = ops.shape(decoder_sequence)[1]
        # We need to handle a rectangular causal mask when doing cached
        # decoding. For generative inference, `decoder_sequence` will
        # generally be length 1, and `cache` will be the full generation length.
        if self_attention_cache is not None:
            input_length = ops.shape(self_attention_cache)[2]

        cache_update_index = (
            0
            if self_attention_cache_update_index is None
            else self_attention_cache_update_index
        )

        causal_mask = compute_causal_mask(
            batch_size, input_length, output_length, cache_update_index
        )

        return (
            ops.minimum(decoder_mask, causal_mask)
            if decoder_mask is not None
            else causal_mask
        )

    def compute_output_shape(self, decoder_sequence_shape):
        """Computes the output shape of the layer.

        Args:
            decoder_sequence_shape: Shape of the decoder sequence input.

        Returns:
            Output shape, which is the same as the input shape.
        """
        return decoder_sequence_shape

    def get_config(self):
        """Returns the config of the layer.

        Returns:
            Dictionary containing the parameters used to initialize this layer.
        """
        config = super().get_config()
        config.update(
            {
                "num_query_heads": self.num_query_heads,
                "intermediate_dim": self.intermediate_dim,
                "moe_intermediate_dim": self.moe_intermediate_dim,
                "shared_expert_intermediate_dim": (
                    self.shared_expert_intermediate_dim
                ),
                "rope_max_wavelength": self.rope_max_wavelength,
                "num_key_value_heads": self.num_key_value_heads,
                "rope_scaling_factor": self.rope_scaling_factor,
                "layer_norm_epsilon": self.layer_norm_epsilon,
                "dropout": self.dropout,
                "use_sliding_window_attention": (
                    self.use_sliding_window_attention
                ),
                "sliding_window_size": self.sliding_window_size,
                "num_experts": self.num_experts,
                "top_k": self.top_k,
                "norm_top_k_prob": self.norm_top_k_prob,
                "decoder_sparse_step": self.decoder_sparse_step,
                "mlp_only_layers": self.mlp_only_layers,
                "output_router_logits": self.output_router_logits,
                "router_aux_loss_coefficient": self.router_aux_loss_coefficient,
            }
        )
        return config



"""Gemma3 model preset configurations."""

# Metadata for loading pretrained model weights.
backbone_presets = {
    "gemma3_1b": {
        "metadata": {
            "description": (
                "1 billion parameter, 26-layer, text-only pretrained "
                "Gemma3 model."
            ),
            "params": 999885952,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_1b/3",
    },
    "gemma3_instruct_1b": {
        "metadata": {
            "description": (
                "1 billion parameter, 26-layer, text-only instruction-tuned "
                "Gemma3 model."
            ),
            "params": 999885952,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_instruct_1b/3",
    },
    "gemma3_4b_text": {
        "metadata": {
            "description": (
                "4 billion parameter, 34-layer, text-only pretrained "
                "Gemma3 model."
            ),
            "params": 3880099328,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_4b_text/2",
    },
    "gemma3_instruct_4b_text": {
        "metadata": {
            "description": (
                "4 billion parameter, 34-layer, text-only instruction-tuned "
                "Gemma3 model."
            ),
            "params": 3880099328,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_instruct_4b_text/3",
    },
    "gemma3_12b_text": {
        "metadata": {
            "description": (
                "12 billion parameter, 48-layer, text-only pretrained "
                "Gemma3 model."
            ),
            "params": 11765788416,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_12b_text/3",
    },
    "gemma3_instruct_12b_text": {
        "metadata": {
            "description": (
                "12 billion parameter, 48-layer, text-only instruction-tuned "
                "Gemma3 model."
            ),
            "params": 11765788416,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_instruct_12b_text/3",
    },
    "gemma3_27b_text": {
        "metadata": {
            "description": (
                "27 billion parameter, 62-layer, text-only pretrained "
                "Gemma3 model."
            ),
            "params": 27009002240,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_27b_text/4",
    },
    "gemma3_instruct_27b_text": {
        "metadata": {
            "description": (
                "27 billion parameter, 62-layer, text-only instruction-tuned "
                "Gemma3 model."
            ),
            "params": 27009002240,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_instruct_27b_text/3",
    },
    "gemma3_4b": {
        "metadata": {
            "description": (
                "4 billion parameter, 34-layer, vision+text pretrained "
                "Gemma3 model."
            ),
            "params": 4299915632,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_4b/1",
    },
    "gemma3_instruct_4b": {
        "metadata": {
            "description": (
                "4 billion parameter, 34-layer, vision+text instruction-tuned "
                "Gemma3 model."
            ),
            "params": 4299915632,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_instruct_4b/1",
    },
    "gemma3_12b": {
        "metadata": {
            "description": (
                "12 billion parameter, 48-layer, vision+text pretrained "
                "Gemma3 model."
            ),
            "params": 12187079280,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_12b/2",
    },
    "gemma3_instruct_12b": {
        "metadata": {
            "description": (
                "12 billion parameter, 48-layer, vision+text instruction-tuned "
                "Gemma3 model."
            ),
            "params": 12187079280,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_instruct_12b/2",
    },
    "gemma3_27b": {
        "metadata": {
            "description": (
                "27 billion parameter, 62-layer, vision+text pretrained "
                "Gemma3 model."
            ),
            "params": 27432062576,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_27b/2",
    },
    "gemma3_instruct_27b": {
        "metadata": {
            "description": (
                "27 billion parameter, 62-layer, vision+text instruction-tuned "
                "Gemma3 model."
            ),
            "params": 27432062576,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_instruct_27b/2",
    },
    "gemma3_270m": {
        "metadata": {
            "description": (
                "270-million parameter(170m embedding,100m transformer params) "
                "model, 18-layer, text-only designed for hyper-efficient AI, "
                "particularly for task-specific fine-tuning."
            ),
            "params": 268098176,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_270m/3",
    },
    "gemma3_instruct_270m": {
        "metadata": {
            "description": (
                "270-million parameter(170m embedding,100m transformer params) "
                "model, 18-layer, text-only,instruction-tuned model designed "
                "for hyper-efficient AI, particularly for task-specific "
                "fine-tuning."
            ),
            "params": 268098176,
            "path": "gemma3",
        },
        "kaggle_handle": "kaggle://keras/gemma3/keras/gemma3_instruct_270m/4",
    },
}



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.backbone import Backbone


@keras_hub_export("keras_hub.models.SegFormerBackbone")
class SegFormerBackbone(Backbone):
    """A Keras model implementing SegFormer for semantic segmentation.

    This class implements the majority of the SegFormer architecture described
    in [SegFormer: Simple and Efficient Design for Semantic Segmentation](https://arxiv.org/abs/2105.15203)
    and based on the TensorFlow implementation
    [from DeepVision](https://github.com/DavidLandup0/deepvision/tree/main/deepvision/models/segmentation/segformer).

    SegFormers are meant to be used with the MixTransformer (MiT) encoder
    family, and use a very lightweight all-MLP decoder head.

    The MiT encoder uses a hierarchical transformer which outputs features at
    multiple scales, similar to that of the hierarchical outputs typically
    associated with CNNs.

    Args:
        image_encoder: `keras.Model`. The backbone network for the model that is
            used as a feature extractor for the SegFormer encoder.
            Should be used with the MiT backbone model
            (`keras_hub.models.MiTBackbone`) which was created
            specifically for SegFormers.
        num_classes: int, the number of classes for the detection model,
            including the background class.
        projection_filters: int, number of filters in the
            convolution layer projecting the concatenated features into
            a segmentation map. Defaults to 256`.

    Example:

    Using the class with a custom `backbone`:

    ```python
    import keras_hub

    backbone = keras_hub.models.MiTBackbone(
        depths=[2, 2, 2, 2],
        image_shape=(224, 224, 3),
        hidden_dims=[32, 64, 160, 256],
        num_layers=4,
        blockwise_num_heads=[1, 2, 5, 8],
        blockwise_sr_ratios=[8, 4, 2, 1],
        max_drop_path_rate=0.1,
        patch_sizes=[7, 3, 3, 3],
        strides=[4, 2, 2, 2],
    )

    segformer_backbone = keras_hub.models.SegFormerBackbone(
        image_encoder=backbone, projection_filters=256)
    ```

    Using the class with a preset `backbone`:

    ```python
    import keras_hub

    backbone = keras_hub.models.MiTBackbone.from_preset("mit_b0_ade20k_512")
    segformer_backbone = keras_hub.models.SegFormerBackbone(
        image_encoder=backbone, projection_filters=256)
    ```

    """

    def __init__(
        self,
        image_encoder,
        projection_filters,
        **kwargs,
    ):
        if not isinstance(image_encoder, keras.layers.Layer) or not isinstance(
            image_encoder, keras.Model
        ):
            raise ValueError(
                "Argument `image_encoder` must be a `keras.layers.Layer` "
                f"instance or `keras.Model`. Received instead "
                f"image_encoder={image_encoder} "
                f"(of type {type(image_encoder)})."
            )

        # === Layers ===
        inputs = keras.layers.Input(shape=image_encoder.input.shape[1:])

        self.feature_extractor = keras.Model(
            image_encoder.inputs, image_encoder.pyramid_outputs
        )

        features = self.feature_extractor(inputs)
        # Get height and width of level one output
        _, height, width, _ = features["P1"].shape

        self.mlp_blocks = []

        for feature_dim, feature in zip(image_encoder.hidden_dims, features):
            self.mlp_blocks.append(
                keras.layers.Dense(
                    projection_filters, name=f"linear_{feature_dim}"
                )
            )

        self.resizing = keras.layers.Resizing(
            height, width, interpolation="bilinear"
        )
        self.concat = keras.layers.Concatenate(axis=-1)
        self.linear_fuse = keras.Sequential(
            [
                keras.layers.Conv2D(
                    filters=projection_filters, kernel_size=1, use_bias=False
                ),
                keras.layers.BatchNormalization(epsilon=1e-5, momentum=0.9),
                keras.layers.Activation("relu"),
            ]
        )

        # === Functional Model ===
        # Project all multi-level outputs onto
        # the same dimensionality and feature map shape
        multi_layer_outs = []
        for index, (feature_dim, feature) in enumerate(
            zip(image_encoder.hidden_dims, features)
        ):
            out = self.mlp_blocks[index](features[feature])
            out = self.resizing(out)
            multi_layer_outs.append(out)

        # Concat now-equal feature maps
        concatenated_outs = self.concat(multi_layer_outs[::-1])

        # Fuse concatenated features into a segmentation map
        seg = self.linear_fuse(concatenated_outs)

        super().__init__(
            inputs=inputs,
            outputs=seg,
            **kwargs,
        )

        # === Config ===
        self.projection_filters = projection_filters
        self.image_encoder = image_encoder

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "projection_filters": self.projection_filters,
                "image_encoder": keras.saving.serialize_keras_object(
                    self.image_encoder
                ),
            }
        )
        return config

    @classmethod
    def from_config(cls, config):
        if "image_encoder" in config and isinstance(
            config["image_encoder"], dict
        ):
            config["image_encoder"] = keras.layers.deserialize(
                config["image_encoder"]
            )
        return super().from_config(config)



"""MobileNetV5 preset configurations."""

backbone_presets = {
    "mobilenetv5_300m_enc_gemma3n": {
        "metadata": {
            "description": (
                "Lightweight 300M-parameter convolutional vision encoder used "
                "as the image backbone for Gemma 3n"
            ),
            "params": 294_284_096,
            "path": "mobilenetv5",
        },
        "kaggle_handle": "kaggle://keras/mobilenetv5/keras/mobilenetv5_300m_enc_gemma3n/1",
    }
}



import types

import keras
import numpy as np

from keras_hub.src.models.mobilenetv5.mobilenetv5_attention import (
    MobileAttention,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_backbone import (
    MobileNetV5Backbone,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_blocks import EdgeResidual
from keras_hub.src.models.mobilenetv5.mobilenetv5_blocks import (
    UniversalInvertedResidual,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_builder import (
    convert_arch_def_to_stackwise,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_layers import ConvNormAct
from keras_hub.src.models.mobilenetv5.mobilenetv5_layers import RmsNorm2d

backbone_cls = MobileNetV5Backbone

MODEL_CONFIGS = {
    "mobilenetv5_300m": {
        "backbone": convert_arch_def_to_stackwise(
            [
                # Stage 0: 128x128 in
                [
                    "er_r1_k3_s2_e4_c128",
                    "er_r1_k3_s1_e4_c128",
                    "er_r1_k3_s1_e4_c128",
                ],
                # Stage 1: 256x256 in
                [
                    "uir_r1_a3_k5_s2_e6_c256",
                    "uir_r1_a5_k0_s1_e4_c256",
                    "uir_r1_a3_k0_s1_e4_c256",
                    "uir_r1_a5_k0_s1_e4_c256",
                    "uir_r1_a3_k0_s1_e4_c256",
                ],
                # Stage 2: 640x640 in
                [
                    "uir_r1_a5_k5_s2_e6_c640",
                    "uir_r1_a5_k0_s1_e4_c640",
                    "uir_r1_a5_k0_s1_e4_c640",
                    "uir_r1_a5_k0_s1_e4_c640",
                    "uir_r1_a5_k0_s1_e4_c640",
                    "uir_r1_a5_k0_s1_e4_c640",
                    "uir_r1_a5_k0_s1_e4_c640",
                    "uir_r1_a5_k0_s1_e4_c640",
                    "uir_r1_a0_k0_s1_e1_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                    "mqa_r1_k3_h12_v2_s1_d64_c640",
                    "uir_r1_a0_k0_s1_e2_c640",
                ],
                # Stage 3: 1280x1280 in
                [
                    "uir_r1_a5_k5_s2_e6_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                    "mqa_r1_k3_h16_s1_d96_c1280",
                    "uir_r1_a0_k0_s1_e2_c1280",
                ],
            ]
        ),
        "stem_size": 64,
        "num_features": 2048,
        "norm_layer": "rms_norm",
        "act_layer": "gelu",
        "use_msfa": True,
        "layer_scale_init_value": 1e-5,
    },
}


def convert_head(task, loader, timm_config):
    pass


def convert_backbone_config(timm_config):
    timm_architecture = timm_config["architecture"]
    if timm_architecture not in MODEL_CONFIGS:
        raise ValueError(f"Unsupported architecture: {timm_architecture}")
    config = MODEL_CONFIGS[timm_architecture].copy()
    backbone_config = config.pop("backbone")
    backbone_config.update(config)
    return backbone_config


def convert_weights(backbone, loader, timm_config):
    def key_exists(key):
        try:
            loader.get_tensor(key)
            return True
        except Exception:
            return False

    def _port_weights(layer, timm_key, transpose_dims=None):
        hf_weight_key = f"{timm_key}.weight"
        if not key_exists(hf_weight_key):
            return
        hook_fn = None
        if transpose_dims:

            def transpose_hook(x, _):
                return np.transpose(x, transpose_dims)

            hook_fn = transpose_hook
        loader.port_weight(
            layer.kernel, hf_weight_key=hf_weight_key, hook_fn=hook_fn
        )
        if layer.bias is not None:
            hf_bias_key = f"{timm_key}.bias"
            if key_exists(hf_bias_key):
                loader.port_weight(
                    layer.bias,
                    hf_weight_key=hf_bias_key,
                )

    def _port_bn(layer, timm_prefix):
        loader.port_weight(layer.gamma, f"{timm_prefix}.weight")
        loader.port_weight(layer.beta, f"{timm_prefix}.bias")
        loader.port_weight(layer.moving_mean, f"{timm_prefix}.running_mean")
        loader.port_weight(layer.moving_variance, f"{timm_prefix}.running_var")

    def _port_rms_norm(layer, timm_prefix):
        loader.port_weight(layer.gamma, f"{timm_prefix}.weight")

    def _port_cna(cna_layer: ConvNormAct, timm_conv_prefix, timm_norm_prefix):
        if isinstance(cna_layer.conv, keras.layers.DepthwiseConv2D):
            _port_weights(
                cna_layer.conv,
                timm_conv_prefix,
                transpose_dims=(2, 3, 0, 1),
            )
        else:
            _port_weights(
                cna_layer.conv,
                timm_conv_prefix,
                transpose_dims=(2, 3, 1, 0),
            )
        if key_exists(f"{timm_norm_prefix}.running_mean"):
            _port_bn(cna_layer.norm, timm_norm_prefix)
        else:
            _port_rms_norm(cna_layer.norm, timm_norm_prefix)

    def _port_attn(attn_layer, attn_prefix):
        _port_weights(
            attn_layer.query_layers[-1],
            f"{attn_prefix}.query.proj",
            (2, 3, 1, 0),
        )
        if len(attn_layer.key_layers) > 1:
            _port_weights(
                attn_layer.key_layers[0],
                f"{attn_prefix}.key.down_conv",
                (2, 3, 0, 1),
            )
            key_norm_layer = attn_layer.key_layers[1]
            if isinstance(key_norm_layer, RmsNorm2d):
                _port_rms_norm(key_norm_layer, f"{attn_prefix}.key.norm")
            else:
                _port_bn(key_norm_layer, f"{attn_prefix}.key.norm")
        _port_weights(
            attn_layer.key_layers[-1], f"{attn_prefix}.key.proj", (2, 3, 1, 0)
        )
        if len(attn_layer.value_layers) > 1:
            _port_weights(
                attn_layer.value_layers[0],
                f"{attn_prefix}.value.down_conv",
                (2, 3, 0, 1),
            )
            value_norm_layer = attn_layer.value_layers[1]
            if isinstance(value_norm_layer, RmsNorm2d):
                _port_rms_norm(value_norm_layer, f"{attn_prefix}.value.norm")
            else:
                _port_bn(value_norm_layer, f"{attn_prefix}.value.norm")
        _port_weights(
            attn_layer.value_layers[-1],
            f"{attn_prefix}.value.proj",
            (2, 3, 1, 0),
        )
        _port_weights(
            attn_layer.output_proj_layers[-2],
            f"{attn_prefix}.output.proj",
            (2, 3, 1, 0),
        )

    stem_layer = backbone.get_layer("conv_stem")
    _port_cna(stem_layer, "conv_stem.conv", "conv_stem.bn")
    block_layers = [
        layer
        for layer in backbone.layers
        if isinstance(
            layer, (EdgeResidual, UniversalInvertedResidual, MobileAttention)
        )
    ]
    block_counter = 0
    for stack_idx in range(len(backbone.stackwise_num_blocks)):
        for block_idx_in_stage in range(
            backbone.stackwise_num_blocks[stack_idx]
        ):
            block = block_layers[block_counter]
            timm_prefix = f"blocks.{stack_idx}.{block_idx_in_stage}"
            if isinstance(block, EdgeResidual):
                _port_cna(
                    block.conv_exp,
                    f"{timm_prefix}.conv_exp",
                    f"{timm_prefix}.bn1",
                )
                _port_cna(
                    block.conv_pwl,
                    f"{timm_prefix}.conv_pwl",
                    f"{timm_prefix}.bn2",
                )
            elif isinstance(block, UniversalInvertedResidual):
                if hasattr(block, "dw_start") and not isinstance(
                    block.dw_start, types.FunctionType
                ):
                    _port_cna(
                        block.dw_start,
                        f"{timm_prefix}.dw_start.conv",
                        f"{timm_prefix}.dw_start.bn",
                    )
                _port_cna(
                    block.pw_exp,
                    f"{timm_prefix}.pw_exp.conv",
                    f"{timm_prefix}.pw_exp.bn",
                )
                if hasattr(block, "dw_mid") and not isinstance(
                    block.dw_mid, types.FunctionType
                ):
                    _port_cna(
                        block.dw_mid,
                        f"{timm_prefix}.dw_mid.conv",
                        f"{timm_prefix}.dw_mid.bn",
                    )
                _port_cna(
                    block.pw_proj,
                    f"{timm_prefix}.pw_proj.conv",
                    f"{timm_prefix}.pw_proj.bn",
                )
                gamma_key = f"{timm_prefix}.layer_scale.gamma"
                if key_exists(gamma_key):
                    loader.port_weight(block.layer_scale.gamma, gamma_key)
            elif isinstance(block, MobileAttention):
                _port_rms_norm(block.norm, f"{timm_prefix}.norm")
                gamma_key = f"{timm_prefix}.layer_scale.gamma"
                if key_exists(gamma_key):
                    loader.port_weight(block.layer_scale.gamma, gamma_key)
                attn_prefix = f"{timm_prefix}.attn"
                _port_attn(block.attn, attn_prefix)
            block_counter += 1
    try:
        msfa_layer = backbone.get_layer("msfa")
        ffn = msfa_layer.ffn
        _port_cna(ffn.pw_exp, "msfa.ffn.pw_exp.conv", "msfa.ffn.pw_exp.bn")
        _port_cna(ffn.pw_proj, "msfa.ffn.pw_proj.conv", "msfa.ffn.pw_proj.bn")
        _port_rms_norm(msfa_layer.norm, "msfa.norm")
    except ValueError:
        pass



import keras
from keras import ops
from keras.src.losses.losses import LossFunctionWrapper


class DepthAnythingLoss(LossFunctionWrapper):
    """Computes the DepthAnything loss between `y_true` & `y_pred`.

    This loss is the Scale-Invariant Logarithmic (SiLog) loss, which is
    widely used for depth estimation tasks.

    See: [Depth Map Prediction from a Single Image using a Multi-Scale Deep Network](https://arxiv.org/abs/1406.2283)

    Args:
        lambd: The weighting factor in the scale-invariant log loss formula.
            Defaults to `0.5`.
        min_depth: Minimum depth value used to filter `y_pred` and `y_true`.
            Defaults to `keras.config.epsilon()`.
        max_depth: Optional maximum depth value used to filter `y_pred` and
            `y_true`. If not specified, there will be no upper bound.
        reduction: Type of reduction to apply to the loss. In almost all cases
            this should be `"sum_over_batch_size"`. Supported options are
            `"sum"`, `"sum_over_batch_size"`, `"mean"`,
            `"mean_with_sample_weight"` or `None`. `"sum"` sums the loss,
            `"sum_over_batch_size"` and `"mean"` sum the loss and divide by the
            sample size, and `"mean_with_sample_weight"` sums the loss and
            divides by the sum of the sample weights. `"none"` and `None`
            perform no aggregation. Defaults to `"sum_over_batch_size"`.
        name: Optional name for the instance.
        dtype: The dtype of the loss's computations. Defaults to `None`, which
            means using `keras.backend.floatx()`. `keras.backend.floatx()` is a
            `"float32"` unless set to different value
            (via `keras.backend.set_floatx()`). If a `keras.DTypePolicy` is
            provided, then the `compute_dtype` will be utilized.
    """

    def __init__(
        self,
        lambd=0.5,
        min_depth=keras.config.epsilon(),
        max_depth=None,
        reduction="sum_over_batch_size",
        name="depth_anything_loss",
        dtype=None,
    ):
        super().__init__(
            silog,
            name=name,
            reduction=reduction,
            dtype=dtype,
            lambd=lambd,
            min_depth=min_depth,
            max_depth=max_depth,
        )


def silog(
    y_true, y_pred, lambd=0.5, min_depth=keras.config.epsilon(), max_depth=None
):
    y_pred = ops.convert_to_tensor(y_pred)
    y_true = ops.convert_to_tensor(y_true, dtype=y_pred.dtype)

    # Apply the valid mask.
    if max_depth is None:
        valid_mask = ops.greater_equal(y_true, min_depth)
    else:
        valid_mask = ops.logical_and(
            ops.greater_equal(y_true, min_depth),
            ops.less_equal(y_true, max_depth),
        )
    y_true = ops.multiply(y_true, valid_mask)
    y_pred = ops.multiply(y_pred, valid_mask)

    diff_log = ops.where(
        valid_mask,
        ops.subtract(ops.log(y_true), ops.log(y_pred)),
        ops.zeros_like(y_true),
    )

    divisor = ops.sum(ops.cast(valid_mask, y_true.dtype), axis=(1, 2, 3))
    mean_power2_diff_log = ops.divide_no_nan(
        ops.sum(ops.power(diff_log, 2), axis=(1, 2, 3)), divisor
    )
    power2_mean_diff_log = ops.power(
        ops.divide_no_nan(ops.sum(diff_log, axis=(1, 2, 3)), divisor), 2
    )
    return ops.sqrt(
        mean_power2_diff_log - ops.multiply(lambd, power2_mean_diff_log)
    )
